"""
Contqains LoggingInitialzer class
"""

import logging
import sys
import tempfile
from datetime import datetime
from pathlib import Path


class LoggingInitializer:
    """
    Applies global logging settings.
    """
    def __init__(self, logfile_dir: Path = None):
        """

        Args:
            logfile_dir: path where logfile is saved.
        """
        if logfile_dir is None:
            self._logfile_dir_temp_obj = tempfile.TemporaryDirectory()
            logfile_dir = Path(self._logfile_dir_temp_obj.name)
        else:
            self._logfile_dir_temp_obj = None
        self._logfile_dir = logfile_dir
        self._log_filename = (
            f"srepkg_log_{datetime.now().strftime('%Y-%m-%d_%H_%M_%S_%f')}.log"
        )

    @staticmethod
    def log_unhandled_exception(exc_type, exc_value, exc_traceback):
        if issubclass(exc_type, KeyboardInterrupt):
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
            return

        logging.getLogger("std_err").critical(
            "Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback)
        )

    def setup(self):
        logging.basicConfig(
            level=logging.DEBUG,
            format="%(asctime)s:%(levelname)s:%(name)s:%(message)s",
            filename=f"{str(self._logfile_dir)}/{self._log_filename}",
            filemode="w",
        )

        console_log_handler_info = {
            "std_err": (logging.DEBUG, sys.stderr),
            "std_out": (logging.DEBUG, sys.stdout),
        }

        for handler_name, handler_info in console_log_handler_info.items():
            logger = logging.getLogger(handler_name)
            handler = logging.StreamHandler(stream=handler_info[1])
            handler.setLevel(level=handler_info[0])
            logger.addHandler(handler)

        sys.excepthook = self.log_unhandled_exception



from pathlib import Path
from typing import NamedTuple
from unittest import mock
import srepkg.repackager_interfaces as rep_int
import srepkg.service_builder as sb
import srepkg.srepkg_builder as s_bldr
from shared_fixtures import dummy_cdir_summary


class BuilderTestCondition(NamedTuple):
    pkg_ref: str
    srepkg_zip_exists: bool
    srepkg_whl_exists: bool


class TestSrepkgBuilder:
    local_test_pkgs_path = (
        Path(__file__).parent.absolute() / "package_test_cases"
    )

    good_test_conditions = [
        BuilderTestCondition(
            pkg_ref=str(local_test_pkgs_path / "testproj"),
            srepkg_zip_exists=True,
            srepkg_whl_exists=True,
        ),
        # BuilderTestCondition(
        #     pkg_ref=str(
        #         local_test_pkgs_path /
        #         'numpy-1.23.2-cp39-cp39-macosx_10_9_x86_64.whl'),
        #     srepkg_zip_exists=False,
        #     srepkg_whl_exists=True
        # )
    ]

    @staticmethod
    def run_test_condition(
        condition: BuilderTestCondition, dist_out_dir: Path
    ):
        srepkg_command = rep_int.SrepkgCommand(
            orig_pkg_ref=condition.pkg_ref, dist_out_dir=str(dist_out_dir)
        )
        service_builder = sb.ServiceBuilder(srepkg_command=srepkg_command)
        osp = service_builder.create_orig_src_preparer()
        construction_dir_summary = osp.prepare()
        srepkg_builder = service_builder.create_srepkg_builder(
            construction_dir_summary=construction_dir_summary
        )
        srepkg_builder.build()

    def test_good_builder_conditions(self, tmp_path_factory):
        for condition in self.good_test_conditions:
            dist_out_dir = tmp_path_factory.mktemp("dist_out")
            self.run_test_condition(
                condition=condition, dist_out_dir=dist_out_dir
            )
            dist_out_contents = list(dist_out_dir.iterdir())
            dist_out_filetypes = [item.suffix for item in dist_out_contents]
            assert (
                ".zip" in dist_out_filetypes
            ) == condition.srepkg_zip_exists
            assert (
                ".whl" in dist_out_filetypes
            ) == condition.srepkg_whl_exists

    # def mock_build_wheel(self):
    #     pass

    def test_init_builder_without_completers(self, dummy_cdir_summary):
        srepkg_builder = s_bldr.SrepkgBuilder(
            construction_dir_summary=dummy_cdir_summary,
            output_dir=Path("dummy"),
        )

    def test_zip_dir_file_exclusion(
        self, tmp_path_factory, dummy_cdir_summary
    ):
        src_path = tmp_path_factory.mktemp("src_path")
        (src_path / "file_to_exclude.txt").touch()
        dest_dir = tmp_path_factory.mktemp("dest_dir")

        zip_name = str(dest_dir / "dest_zip.zip")
        s_bldr.SrepkgSdistWriter(
            orig_pkg_summary=dummy_cdir_summary, dist_out_dir=dest_dir
        ).write_to_zip(
            zip_name=zip_name,
            # src_path=src_path,
            # exclude_paths=[src_path / "file_to_exclude.txt"],
        )

    def test_copy_ready_comps_with_dir_as_content(
        self, dummy_cdir_summary, tmp_path_factory
    ):
        srepkg_root = tmp_path_factory.mktemp("srepkg_root")
        cdir_summary = dummy_cdir_summary
        cdir_summary.srepkg_root = srepkg_root

        repackaging_components = tmp_path_factory.mktemp("components_dir")
        (repackaging_components / "wheel_completer_components").mkdir()
        (repackaging_components / "wheel_completer_templates").mkdir()

        dummy_content_path = repackaging_components / "content"
        dummy_content_path.mkdir()

        with mock.patch(
            "srepkg.srepkg_builder.SrepkgWheelCompleter._props",
            new_callable=mock.PropertyMock,
        ) as mock_props:
            mock_props.return_value = s_bldr.CompleterProperties(
                components_dir=repackaging_components,
                templates_dir=Path("dummy"),
                dist_writer_type=s_bldr.SrepkgWheelWriter,
            )

            wheel_completer = s_bldr.SrepkgWheelCompleter(
                orig_pkg_summary=cdir_summary,
                dist_out_dir=Path("dummy"),
                repkg_components=repackaging_components,
            )

            wheel_completer._copy_ready_components()



import subprocess
import sys
from pathlib import Path


def entry_funct():
    entry_command = Path(__file__).name[:-3]
    pkg_args = sys.argv[1:]
    venv_bin = Path(__file__).parent.parent.absolute() / "srepkg_venv" / "bin"
    subprocess.call([venv_bin / entry_command, *pkg_args])



import hashlib
import sys
import tempfile
from pathlib import Path
import srepkg.dist_builder_sub_process as dbs
import srepkg.error_handling.custom_exceptions as ce
import srepkg.utils.logged_err_detecting_subprocess as leds
import srepkg.utils.pkg_type_identifier as pti


class DistBuilder:

    def __init__(
        self,
        distribution: str,
        source_dir: Path,
        output_directory: Path,
        std_out_file: Path = tempfile.TemporaryFile(),
        std_err_file: Path = tempfile.TemporaryFile(),
    ):
        self._distribution = distribution
        self._source_dir = source_dir
        self._output_directory = output_directory
        self._output_directory.mkdir(parents=True, exist_ok=True)
        self._std_out_file = std_out_file
        self._std_err_file = std_err_file

    @staticmethod
    def _calc_md5(file_path: Path):
        hash_md5 = hashlib.md5()
        with file_path.open(mode="rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()

    @property
    def _files_in_dest_dir(self):
        return [
            item
            for item in self._output_directory.iterdir()
            if not item.is_dir()
        ]

    def build(self):
        orig_dest_checksums = [
            self._calc_md5(item) for item in self._files_in_dest_dir
        ]

        leds.LoggedErrDetectingSubprocess(
            cmd=[
                sys.executable,
                dbs.__file__,
                self._distribution,
                str(self._source_dir),
                str(self._output_directory),
            ],
            gen_logger_name=__name__,
            std_out_logger_name="std_out",
            std_err_logger_name="std_err",
            default_exception=ce.BuildSubprocessError,
        ).run()

        new_dist_files = [
            item
            for item in self._files_in_dest_dir
            if (
                self._calc_md5(item) not in orig_dest_checksums
                and (
                    pti.PkgRefIdentifier(str(item)).identify_for_osp_dispatch()
                    == pti.PkgRefType.LOCAL_DIST
                )
            )
        ]
        assert len(new_dist_files) == 1

        return new_dist_files[0]



import logging
import requests
import subprocess
import sys
from enum import Enum, auto
from pathlib import Path
from urllib.parse import urlparse
import srepkg.error_handling.error_messages as em
import srepkg.utils.dist_archive_file_tools as cdi
import srepkg.utils.logged_err_detecting_subprocess as leds


class PkgRefType(Enum):
    LOCAL_SRC_NONGIT = auto()
    LOCAL_WHEEL = auto()
    LOCAL_SDIST = auto()
    LOCAL_DIST = auto()
    PYPI_PKG = auto()
    GIT_REPO = auto()
    UNKNOWN = auto()
    MULTIPLE_POSSIBLE = auto()


class PkgRefIdentifier:
    def __init__(self, orig_pkg_ref: str):
        self._orig_pkg_ref = orig_pkg_ref

    def is_local_git_repo(self):
        # choose to NOT wrape this in LoggedErrorDetectingSubprocess b/c we
        # are OK if subprocess return code != 0
        logging.getLogger(__name__).debug(
            "Running git status as subprocess to check if original pkg is a "
            "local git repo"
        )
        p = subprocess.run(
            ["git", "-C", self._orig_pkg_ref, "status"],
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
            universal_newlines=True,
        )

        for line in p.stdout.strip().split("\n"):
            logging.getLogger(__name__).debug(line)
        for line in p.stderr.strip().split("\n"):
            logging.getLogger(__name__).debug(line)

        return (
            (p.returncode == 0)
            and Path(self._orig_pkg_ref).is_dir()
            and (
                ".git"
                in [
                    item.name
                    for item in list(Path(self._orig_pkg_ref).iterdir())
                ]
            )
        )

    def is_local_src_non_git(self):
        return Path(self._orig_pkg_ref).is_dir() and (
            not self.is_local_git_repo()
        )

    def is_local_wheel(self):
        return (
            (not Path(self._orig_pkg_ref).is_dir())
            and Path(self._orig_pkg_ref).exists()
            and (
                cdi.ArchiveIdentifier().id_file_type(Path(self._orig_pkg_ref))
                == cdi.ArchiveFileType.WHL
            )
        )

    def is_local_sdist(self):
        return (
            (not Path(self._orig_pkg_ref).is_dir())
            and Path(self._orig_pkg_ref).exists()
            and (
                cdi.ArchiveIdentifier().id_file_type(Path(self._orig_pkg_ref))
                != cdi.ArchiveFileType.UNKNOWN
            )
            and (Path(self._orig_pkg_ref).suffix != ".whl")
        )

    def is_pypi_pkg(self):
        response = requests.get(
            "https://pypi.python.org/pypi/{}/json".format(self._orig_pkg_ref)
        )
        return response.status_code == 200

    def is_github_repo(self):
        url_parsed_ref = urlparse(self._orig_pkg_ref)
        return url_parsed_ref.netloc == "github.com" and (
            len(url_parsed_ref.path.split("/")) > 1
        )

    def is_git_repo(self):
        return self.is_local_git_repo() or self.is_github_repo()

    def _check_all_types(self):
        return {
            # PkgRefType.LOCAL_SRC_GIT: self.is_local_git_repo(),
            PkgRefType.LOCAL_SRC_NONGIT: self.is_local_src_non_git(),
            PkgRefType.LOCAL_SDIST: self.is_local_sdist(),
            PkgRefType.LOCAL_WHEEL: self.is_local_wheel(),
            PkgRefType.PYPI_PKG: self.is_pypi_pkg(),
            PkgRefType.GIT_REPO: self.is_git_repo(),
        }

    def identify(self) -> PkgRefType:
        pkg_check_results = self._check_all_types()
        matching_items = [
            item[0] for item in pkg_check_results.items() if item[1] is True
        ]
        num_matches = len(matching_items)

        if num_matches == 0:
            return PkgRefType.UNKNOWN
            # sys.exit(em.PkgIdentifierError.PkgNotFound.msg)

        if num_matches > 1:
            return PkgRefType.MULTIPLE_POSSIBLE
            # sys.exit(em.PkgIdentifierError.MultiplePotentialPackages.msg)

        return matching_items[0]

    def identify_for_osp_dispatch(self):
        gen_pkg_ref_id = self.identify()
        if (gen_pkg_ref_id == PkgRefType.LOCAL_SDIST) or (
            gen_pkg_ref_id == PkgRefType.LOCAL_WHEEL
        ):
            return PkgRefType.LOCAL_DIST
        return gen_pkg_ref_id



from enum import Enum
from typing import NamedTuple


class ErrorMsg(NamedTuple):
    msg: str


class OrigPkgError(ErrorMsg, Enum):
    PkgPathNotFound = ErrorMsg(msg="Original package path not found")
    SetupPyNotFound = ErrorMsg(
        msg="No setup.py file found. srepkg requires a setup.py file with call"
        "to setuptools.setup()"
    )
    NoCSE = ErrorMsg(msg="No console script entry points found")


class SetupFileReaderError(ErrorMsg, Enum):
    SetupCfgReadError = ErrorMsg(msg="Unable to read or parse setup.cfg")
    UnsupportedSetupFileType = ErrorMsg(msg="Unsupported setup file type")


class SrepkgBuilderError(ErrorMsg, Enum):
    OrigPkgPathNotFound = ErrorMsg(msg="Original package path not found")
    DestPkgPathExits = ErrorMsg(
        msg="Intended Srepkg destination path already exists"
    )
    ControlComponentsNotFound = ErrorMsg(
        msg="Error when attempting to copy sub-package "
        "srepkg_control_components. Sub-package not found"
    )
    ControlComponentsExist = ErrorMsg(
        msg="Error when attempting to copy sub-package "
        "srepkg_control_components. Destination path already exists."
    )
    FileNotFoundForCopy = ErrorMsg(
        msg="Error when attempting to copy. Source file not found."
    )
    CopyDestinationPathExists = ErrorMsg(
        msg="Error when attempting to copy. Destination path already exists"
    )


class PkgRetrieverError(ErrorMsg, Enum):
    InvalidPkgRef = ErrorMsg(
        msg="Package reference provided to srepkg is not recognized as a valid "
        "package."
    )


class PkgIdentifierError(ErrorMsg, Enum):
    PkgNotFound = ErrorMsg(
        msg="Package reference provided to srepkg is not recognized as a valid "
        "package."
    )
    MultiplePotentialPackages = ErrorMsg(
        msg="Package reference is consistent with more than one package."
    )


class ConstructionDirError(ErrorMsg, Enum):
    MissingORigPkgContent = ErrorMsg(
        msg="Srepkg construction directory does not contain any original "
        "content. Expect to find a wheel or source distribution, but "
        "neither found."
    )
    MultiplePackagesPresent = ErrorMsg(
        msg="Srepkg build directory contains distributions from multiple "
        "packages and/or versions."
    )
    TargetDistTypeNotSupported = ErrorMsg(
        msg="Srepkg does not support conversion of original package to this "
        "type"
    )
    NoSDistForWheelConstruction = ErrorMsg(
        msg="Conversion of Sdist to Wheel is requested, but no Sdist found in "
        "srepkg construction directory."
    )


class EntryPontExtractorError(ErrorMsg, Enum):
    NoEntryPointsFile = ErrorMsg(
        msg="No entry_points.txt file found in wheel."
    )


class ArchiveFileError(ErrorMsg, Enum):
    UnsupportedFileType = ErrorMsg(
        msg="Extraction of item at provided path is not supported."
    )



import logging
from contextlib import contextmanager
from yaspin import yaspin


class YaspinLogUpdater:
    def __init__(self):
        self.text = None
        self.spinner = yaspin().runner

    def ok(self, ok_msg: str):
        self.spinner.ok(ok_msg)


@contextmanager
def yaspin_log_updater(msg, logger: logging.Logger):
    updater = YaspinLogUpdater()

    logger.info(msg)
    with updater.spinner as sp:
        sp.text = msg
        yield updater
        sp.ok("✅")



import pytest
import srepkg.command_input as ci
import srepkg.error_handling.custom_exceptions as ce


class TestCommandInput:

    local_src_pkg_ref = "./local_pkg_ref"
    user_srepkg_name = "user_srepkg_name"
    user_construction_dir = "user_construction_dir"
    user_dist_out_dir = "user_dist_out_dir"
    extra_arg = "extra_arg"

    def test_zero_args(self, capsys):
        with pytest.raises(SystemExit):
            srepkg_args = ci.SrepkgCommandLine().get_args([])

    def test_one_arg(self):
        args = ci.SrepkgCommandLine().get_args([self.local_src_pkg_ref])
        assert args.orig_pkg_ref == self.local_src_pkg_ref
        assert args.srepkg_name is None

    def test_valid_custom_name(self):
        args = ci.SrepkgCommandLine().get_args(
            [self.local_src_pkg_ref, "-n", self.user_srepkg_name]
        )
        assert args.orig_pkg_ref == self.local_src_pkg_ref
        assert args.srepkg_name == self.user_srepkg_name

    def test_custom_construction_dir(self):
        args = ci.SrepkgCommandLine().get_args(
            [self.local_src_pkg_ref, "-c", self.user_construction_dir]
        )
        assert args.orig_pkg_ref == self.local_src_pkg_ref
        assert args.construction_dir == self.user_construction_dir

    def test_custom_dist_out_dir(self):
        args = ci.SrepkgCommandLine().get_args(
            [self.local_src_pkg_ref, "-d", self.user_dist_out_dir]
        )
        assert args.orig_pkg_ref == self.local_src_pkg_ref
        assert args.dist_out_dir == self.user_dist_out_dir

    def test_too_many_args(self, capsys):
        with pytest.raises(SystemExit):
            ci.SrepkgCommandLine().get_args(
                [
                    self.local_src_pkg_ref,
                    "-n",
                    self.user_srepkg_name,
                    self.extra_arg,
                ]
            )

    def test_pypi_version_and_git_ref(self):
        with pytest.raises(ce.PkgVersionWithCommitRef):
            ci.SrepkgCommandLine().get_args(
                [
                    self.local_src_pkg_ref,
                    "-g",
                    "dummy_git_ref",
                    "-r",
                    "dummy_pypi_version",
                ]
            )



import subprocess
from pathlib import Path
from typing import List

# from .error_messages import SetupFileReaderError
# from .error_messages import PkgRetrieverError
from .error_messages import ConstructionDirError
from .error_messages import ArchiveFileError


# class UnsupportedSetupFileType(Exception):
#     def __init__(
#             self,
#             file_name: str,
#             msg=SetupFileReaderError.UnsupportedSetupFileType.msg,
#     ):
#         self._file_name = file_name
#         self._msg = msg
#         super().__init__(self._msg)
#
#     def __str__(self):
#         return f"{self._file_name} -> {self._msg}"


# class InvalidPkgRef(Exception):
#     def __init__(
#             self,
#             pkg_ref: str,
#             msg=PkgRetrieverError.InvalidPkgRef.msg):
#         self._pkg_ref = pkg_ref
#         self._msg = msg
#
#     def __str__(self):
#         return f"{self._pkg_ref} -> {self._msg}"


class MissingOrigPkgContent(Exception):
    def __init__(
        self,
        orig_dists_path: str,
        msg=ConstructionDirError.MissingORigPkgContent.msg,
    ):
        self._orig_dists_path = orig_dists_path
        self._msg = msg

    def __str__(self):
        return f"{self._orig_dists_path} -> {self._msg}"


class UnsupportedCompressionType(Exception):
    def __init__(
        self,
        unsupported_file: str,
        msg=ArchiveFileError.UnsupportedFileType.msg,
    ):
        self._unsupported_file = unsupported_file
        self._msg = msg

    def __str__(self):
        return f"{self._unsupported_file} -> {self._msg}"


class MultiplePackagesPresent(Exception):
    def __init__(
        self,
        dists_info: List,
        msg=ConstructionDirError.MultiplePackagesPresent.msg,
    ):
        self._dists_info = dists_info
        self._msg = msg

    def __str__(self):
        return f"{self._dists_info} -> {self._msg}"


class TargetDistTypeNotSupported(Exception):
    def __init__(
        self,
        unsupported_dist_type,
        msg=ConstructionDirError.TargetDistTypeNotSupported.msg,
    ):
        self._unsupported_dist_type = unsupported_dist_type
        self._msg = msg

    def __str__(self):
        return f"{self._unsupported_dist_type} -> {self._msg}"


class NoSDistForWheelConstruction(Exception):
    def __init__(
        self,
        construction_dir: Path,
        msg=ConstructionDirError.NoSDistForWheelConstruction.msg,
    ):
        self._construction_dir = construction_dir
        self._msg = msg

    def __str__(self):
        return f"{str(self._construction_dir)} -> {self._msg}"


class NoEntryPtsTxtFile(Exception):
    def __init__(
        self, whl_path: Path, msg="No entry_points.txt file found in wheel."
    ):
        self._whl_path = whl_path
        self._msg = msg

    def __str__(self):
        return f"{str(self._whl_path)} -> {self._msg}"


class MultipleEntryPtsTxtFiles(Exception):
    def __init__(
        self,
        whl_path: Path,
        msg="Multiple entry_points.txt files found in wheel",
    ):
        self._whl_path = whl_path
        self._msg = msg

    def __str__(self):
        return f"{str(self._whl_path)} -> {self._msg}"


class NoConsoleScriptEntryPoints(Exception):
    def __init__(
        self,
        whl_path: Path,
        msg="No console script entry points found in wheel",
    ):
        self._whl_path = whl_path
        self._msg = msg

    def __str__(self):
        return f"{str(self._whl_path)} -> {self._msg}"


# TODO Make subprocess exception base class and have others inherit from it


class GitCheckoutError(Exception):
    def __init__(
        self,
        sub_process: subprocess.CompletedProcess,
        msg="Error when attempting to checkout git commit in a " "subprocess",
    ):
        self._sub_process = sub_process
        self._msg = msg

    def __str__(self):
        return f"{str(self._sub_process)} -> {self._msg}"


class GitCloneError(Exception):
    def __init__(
        self,
        sub_process: subprocess.CompletedProcess,
        msg="Error when cloning original package Github repo via "
        "subprocess",
    ):

        self._sub_process = sub_process
        self._msg = msg

    def __str__(self):
        return f"{str(self._sub_process)} -> {self._msg}"


class WheelUnpackError(Exception):
    def __init__(
        self,
        sub_process: subprocess.CompletedProcess,
        msg="Error when unpacking wheel via subprocess",
    ):

        self._sub_process = sub_process
        self._msg = msg

    def __str__(self):
        return f"{str(self._sub_process)} -> {self._msg}"


class UnusableGitCommitRef(Exception):
    def __init__(
        self,
        commit_ref: str,
        msg="Git commit ref can only be provided when original package ref"
        "is either a Github repo or a local git repository",
    ):
        self._commit_ref = commit_ref
        self._msg = msg

    def __str__(self):
        return f"{str(self._commit_ref)} -> {self._msg}"


class UnusableVersionArgument(Exception):
    def __init__(
        self,
        version_argument: str,
        msg="Version argument can only be provided when original package "
        "ref isa PyPI package",
    ):
        self._version_argument = version_argument
        self._msg = msg

    def __str__(self):
        return f"{str(self._version_argument)} -> {self._msg}"


class PkgVersionWithCommitRef(Exception):
    def __init__(
        self,
        commit_ref: str,
        pkg_version: str,
        msg="Cannot specify package version and git commit ref in same "
        "command",
    ):
        self._commit_ref = commit_ref
        self._pkg_version = pkg_version
        self._msg = msg

    def __str__(self):
        return (
            f"ORIG_PKG_VERSION: {str(self._pkg_version)},"
            f"GIT_COMMIT_REF: {str(self._commit_ref)} -> {self._msg}"
        )


class BuildSubprocessError(Exception):
    def __init__(
        self,
        sub_process: subprocess.CompletedProcess,
        msg="Error occurred when running subprocess intended build a sdist"
        " or wheel.",
    ):
        self._sub_process = sub_process
        self._msg = msg

    def __str__(self):
        return f"{str(self._sub_process)} -> {self._msg}"



import logging
import shutil
from packaging.tags import Tag
from packaging.utils import parse_wheel_filename
from pathlib import Path

import srepkg.dist_builder as db
import srepkg.error_handling.custom_exceptions as ce
import srepkg.orig_src_preparer_interfaces as osp_int
import srepkg.utils.logged_err_detecting_subprocess as leds

import inner_pkg_installer.yaspin_updater as yu


class DistProviderFromSrc(osp_int.DistProviderInterface):

    def __init__(self, src_path: Path, dest_path: Path):
        self._src_path = src_path
        self._dest_path = dest_path

    def run(self):
        with yu.yaspin_log_updater(
            msg="Building original package wheel from source code",
            logger=logging.getLogger(__name__),
        ):
            wheel_path = db.DistBuilder(
                distribution="wheel",
                source_dir=self._src_path,
                output_directory=self._dest_path,
            ).build()

        wheel_filename = wheel_path.name
        name, version, bld, tags = parse_wheel_filename(wheel_filename)

        if Tag("py3", "none", "any") not in tags:
            with yu.yaspin_log_updater(
                msg="Building original package Sdist from source code",
                logger=logging.getLogger(__name__),
            ):
                db.DistBuilder(
                    distribution="sdist",
                    source_dir=self._src_path,
                    output_directory=self._dest_path,
                ).build()


class DistProviderFromGitRepo(DistProviderFromSrc):
    def __init__(
        self,
        src_path: Path,
        dest_path: Path,
        git_ref: str = None,
        version_command=None,
    ):
        super().__init__(src_path, dest_path)
        self._git_ref = git_ref
        self._version_command = version_command

    def _checkout_commit_ref(self):
        if self._git_ref:
            with yu.yaspin_log_updater(
                msg=f"Checking out {self._git_ref}",
                logger=logging.getLogger(__name__),
            ):
                leds.LoggedErrDetectingSubprocess(
                    cmd=["git", "checkout", self._git_ref],
                    gen_logger_name=__name__,
                    std_out_logger_name="std_out",
                    std_err_logger_name="std_err",
                    default_exception=ce.GitCheckoutError,
                    cwd=self._src_path,
                ).run()

    def run(self):
        self._checkout_commit_ref()
        super().run()


class DistCopyProvider(osp_int.DistProviderInterface):

    def __init__(self, src_path: Path, dest_path: Path):
        self._src_path = src_path
        self._dest_path = dest_path

    def run(self):
        with yu.yaspin_log_updater(
            msg=(
                f"Copying {self._src_path.name} into "
                f"srepkg build directory"
            ),
            logger=logging.getLogger(__name__),
        ):
            shutil.copy2(self._src_path, self._dest_path)

        logging.getLogger(__name__).info(
            f"Copied {self._src_path} into {self._dest_path}"
        )



import pytest
from pathlib import Path
from typing import NamedTuple
import srepkg.utils.dist_archive_file_tools as daft
import srepkg.utils.cd_context_manager as cdcm
import srepkg.utils.pkg_type_identifier as pti
import srepkg.error_handling.custom_exceptions as ce


def test_dir_change_to(tmp_path):
    cwd = Path.cwd()
    assert cwd != tmp_path

    with cdcm.dir_change_to(tmp_path):
        assert Path.cwd() == tmp_path

    assert Path.cwd() == cwd


class ExpectedFileDistType(NamedTuple):
    file_name: str
    file_type: daft.ArchiveFileType
    dist_type: daft.ArchiveDistType


class TestArchiveIdentifier:
    expected_id_info = [
        ExpectedFileDistType(
            file_name="testproj-0.0.0.tar.gz",
            file_type=daft.ArchiveFileType.TAR_GZ,
            dist_type=daft.ArchiveDistType.SDIST,
        ),
        ExpectedFileDistType(
            file_name="testproj-0.0.0.zip",
            file_type=daft.ArchiveFileType.ZIP,
            dist_type=daft.ArchiveDistType.SDIST,
        ),
        ExpectedFileDistType(
            file_name="testproj-0.0.0-py3-none-any.whl",
            file_type=daft.ArchiveFileType.WHL,
            dist_type=daft.ArchiveDistType.WHEEL,
        ),
        ExpectedFileDistType(
            file_name="testproj-0.0.0-not-a-distribution.py",
            file_type=daft.ArchiveFileType.UNKNOWN,
            dist_type=daft.ArchiveDistType.UNKNOWN,
        ),
    ]

    identifier = daft.ArchiveIdentifier()
    test_cases_path = Path(__file__).parent.absolute() / "package_test_cases"

    def run_file_and_dist_id_test(self, info: ExpectedFileDistType):
        assert (
            self.identifier.id_file_type(self.test_cases_path / info.file_name)
            == info.file_type
        )
        assert (
            self.identifier.id_dist_type(self.test_cases_path / info.file_name)
            == info.dist_type
        )

    def test_all_expected_info(self):
        for test_case in self.expected_id_info:
            self.run_file_and_dist_id_test(test_case)


class TestCompressedFileExtractor:
    test_cases_path = Path(__file__).parent.absolute() / "package_test_cases"

    archive_filenames = [
        "testproj-0.0.0.tar.gz",
        "testproj-0.0.0.zip",
        "testproj-0.0.0-py3-none-any.whl",
    ]

    extractor = daft.CompressedFileExtractor()

    def run_expected_good_extraction(self, file_name: str, output_dir: Path):
        self.extractor.extract(self.test_cases_path / file_name, output_dir)
        assert (output_dir / "testproj-0.0.0").exists()

    def test_expected_good_extractions(self, tmp_path):
        for file_name in self.archive_filenames:
            self.run_expected_good_extraction(file_name, tmp_path)

    def test_bad_extraction(self, tmp_path):
        with pytest.raises(ce.UnsupportedCompressionType):
            self.extractor.extract(
                self.test_cases_path / "testproj-0.0.0-not-a-distribution.py",
                tmp_path,
            )


class BrokenPkgRefIdentifier(pti.PkgRefIdentifier):
    def _check_all_types(self):
        return {
            pti.PkgRefType.LOCAL_SRC_NONGIT: True,
            pti.PkgRefType.LOCAL_SDIST: True,
            pti.PkgRefType.LOCAL_WHEEL: True,
            pti.PkgRefType.PYPI_PKG: True,
            pti.PkgRefType.GIT_REPO: True,
        }


class TestPkgRefIdentifier:
    local_test_pkgs_path = (
        Path(__file__).parent.absolute() / "package_test_cases"
    )

    def test_bad_pkg_ref(self):
        pkg_ref_identifier = pti.PkgRefIdentifier("bad_ref")
        pkg_ref_type = pkg_ref_identifier.identify()
        assert pkg_ref_type == pti.PkgRefType.UNKNOWN
        # with pytest.raises(SystemExit):
        #     pkg_ref_identifier.identify()

    def test_multiple_possible_types(self):
        pkg_ref_identifier = BrokenPkgRefIdentifier(
            orig_pkg_ref=str(self.local_test_pkgs_path / "testproj")
        )
        pkg_ref_type = pkg_ref_identifier.identify()
        assert pkg_ref_type == pti.PkgRefType.MULTIPLE_POSSIBLE



import pkginfo
from pathlib import Path
from typing import Callable
import srepkg.repackager_data_structs as re_ds
from shared_fixtures import dummy_cdir_args


# These tests are not exhaustive. Just run conditions missed by other tests.
class TestSrepkgBuilderDataStructs:
    local_test_pkgs_path = (
        Path(__file__).parent.absolute() / "package_test_cases"
    )

    @staticmethod
    def mock_dist_info(
        dist_path: Path, dist_type: Callable[..., pkginfo.Distribution]
    ):
        dist_info = re_ds.DistInfo(
            path=dist_path, dist_obj=dist_type(str(dist_path))
        )
        return dist_info

    @staticmethod
    def run_all_property_getters(obj):
        property_getters = [
            item
            for item in dir(obj)
            if isinstance(getattr(type(obj), item, None), property)
        ]
        for getter in property_getters:
            getattr(obj, getter)

    def test_construction_dir_summary_init_with_dists(self, dummy_cdir_args):
        dist_info = self.mock_dist_info(
            dist_path=self.local_test_pkgs_path
            / "testproj-0.0.0-py3-none-any.whl",
            dist_type=pkginfo.Wheel,
        )
        construction_dir_summary = re_ds.ConstructionDirSummary(
            **dummy_cdir_args, dists=[dist_info]
        )
        assert len(construction_dir_summary.dists) == 1
        assert construction_dir_summary.has_platform_indep_wheel
        assert construction_dir_summary.has_wheel
        assert not construction_dir_summary.has_sdist
        assert (
            construction_dir_summary.src_for_srepkg_wheel.name
            == "testproj-0.0.0-py3-none-any.whl"
        )

    def test_construction_dir_summary_init_with_dists_and_entry_pts(
        self, dummy_cdir_args
    ):

        dist_info = self.mock_dist_info(
            dist_path=self.local_test_pkgs_path
            / "testproj-0.0.0-py3-none-any.whl",
            dist_type=pkginfo.Wheel,
        )
        entry_pt = re_ds.CSEntryPoint(
            command="test", module="test", attr="test"
        )
        construction_dir_summary = re_ds.ConstructionDirSummary(
            **dummy_cdir_args,
            dists=[dist_info],
            entry_pts=re_ds.PkgCSEntryPoints([entry_pt]),
        )
        assert len(construction_dir_summary.dists) == 1
        assert construction_dir_summary.has_platform_indep_wheel
        assert construction_dir_summary.has_wheel
        assert not construction_dir_summary.has_sdist
        assert (
            construction_dir_summary.src_for_srepkg_wheel.name
            == "testproj-0.0.0-py3-none-any.whl"
        )

    def test_wheel_path_getter_without_wheel(self, dummy_cdir_args):
        construction_dir_summary = re_ds.ConstructionDirSummary(
            **dummy_cdir_args
        )
        self.run_all_property_getters(construction_dir_summary)

    def test_sdist_path_getter_with_sdist(self, dummy_cdir_args):
        dist_info = self.mock_dist_info(
            dist_path=self.local_test_pkgs_path / "testproj-0.0.0.tar.gz",
            dist_type=pkginfo.SDist,
        )
        construction_dir_summary = re_ds.ConstructionDirSummary(
            **dummy_cdir_args, dists=[dist_info]
        )
        self.run_all_property_getters(construction_dir_summary)



import pytest
import srepkg.dist_provider as d_prov
from pathlib import Path
from shared_fixtures import sample_pkgs, tmp_construction_dir


class TestDistProvider:

    @pytest.mark.parametrize(
        "provider_constructor, commit_ref, src_path, num_orig_pkgs",
        [
            (d_prov.DistProviderFromSrc, None, "testproj", 1),
            (d_prov.DistProviderFromSrc, None, "tproj_non_pure_py", 2),
            (d_prov.DistCopyProvider, None, "testproj_targz", 1),
        ],
    )
    def test_provider_sources(
        self,
        provider_constructor,
        commit_ref,
        src_path,
        tmp_construction_dir,
        sample_pkgs,
        num_orig_pkgs,
    ):
        constructor_args = {
            "src_path": Path(getattr(sample_pkgs, src_path)),
            "dest_path": tmp_construction_dir.orig_pkg_dists,
        }

        dist_provider = provider_constructor(**constructor_args)
        dist_provider.run()
        assert (
            len(list(tmp_construction_dir.orig_pkg_dists.iterdir()))
            == num_orig_pkgs
        )



import configparser
import contextlib
import os
import shutil
import tempfile
import zipfile
from pathlib import Path
from typing import Callable

from wheel.cli.pack import pack
from wheel.cli.unpack import unpack


def silent_unpack(path: str, dest: str):
    """Unpack a .whl file silently."""
    with open(os.devnull, 'w') as devnull:
        with contextlib.redirect_stdout(devnull), contextlib.redirect_stderr(devnull):
            unpack(path, dest)

def silent_pack(directory: str, dest_dir: str, build_number: str | None):
    """Pack a directory into a .whl file silently."""
    with open(os.devnull, 'w') as devnull:
        with contextlib.redirect_stdout(devnull), contextlib.redirect_stderr(devnull):
            pack(directory, dest_dir, build_number)

class WheelDistInfo:
    def __init__(self, wheel_path: Path):
        self.wheel_path = wheel_path
        with zipfile.ZipFile(wheel_path) as zip_file:
            wheel_relative_paths = zip_file.namelist()
        self.dist_info_dirname = self.get_dist_info_dirname(
            wheel_relative_paths=wheel_relative_paths
        )
        self.child_paths = [
            item
            for item in wheel_relative_paths
            if item.startswith(self.dist_info_dirname)
        ]

    @staticmethod
    def get_dist_info_dirname(wheel_relative_paths: list[str]) -> str:
        dist_info_dirs = {
            path.split("/", 1)[0]
            for path in wheel_relative_paths
            if path.split("/", 1)[0].endswith(".dist-info")
        }

        if len(dist_info_dirs) == 0:
            raise ValueError("No dist-info directory")
        if len(dist_info_dirs) > 1:
            raise ValueError("Multiple dist-info directories found")

        return next(iter(dist_info_dirs))

    @property
    def has_entry_points_txt(self) -> bool:
        return f"{self.dist_info_dirname}/entry_points.txt" in self.child_paths

    @property
    def entry_points_txt_rel_path(self) -> Path | None:
        if self.has_entry_points_txt:
            return Path(self.dist_info_dirname) / "entry_points.txt"

    def get_entry_points_config(self) -> configparser.ConfigParser | None:
        if self.has_entry_points_txt:
            with zipfile.ZipFile(self.wheel_path) as zip_file:
                temp_dir = tempfile.TemporaryDirectory()
                zip_file.extract(
                    member=str(self.entry_points_txt_rel_path),
                    path=temp_dir.name,
                )

                config = configparser.ConfigParser()
                config.read(
                    f"{temp_dir.name}/{self.dist_info_dirname}/entry_points.txt"
                )
            return config

    @property
    def console_script_names(self) -> list[str]:
        if not self.has_entry_points_txt:
            return []
        entry_points_config = self.get_entry_points_config()
        if not entry_points_config.has_section("console_scripts"):
            return []
        else:
            return list(dict(entry_points_config["console_scripts"]).keys())

    @property
    def has_console_script_name_with_dash(self) -> bool:
        return any(["-" in item for item in self.console_script_names])


class WheelEntryPointsModifier:
    def __init__(self, wheel_path: Path):
        self.wheel_path = wheel_path
        self.wheel_dist_info = WheelDistInfo(wheel_path=wheel_path)

    @staticmethod
    def fix_console_script_names(
        entry_points_config: configparser.ConfigParser,
    ):
        for orig_script_name, func in entry_points_config[
            "console_scripts"
        ].items():
            if "-" in orig_script_name:
                new_name = orig_script_name.replace("-", "_")
                entry_points_config.remove_option(
                    "console_scripts", orig_script_name
                )
                entry_points_config.set("console_scripts", new_name, func)

    def modify_entry_points_txt(
        self,
        unpacked_wheel_path: Path,
        config_modifier: Callable[[configparser.ConfigParser], None],
    ):
        entry_points_txt_path = (
            unpacked_wheel_path
            / self.wheel_dist_info.dist_info_dirname
            / "entry_points.txt"
        )
        entry_points_config = configparser.ConfigParser()
        entry_points_config.read(entry_points_txt_path)
        config_modifier(entry_points_config)
        with entry_points_txt_path.open(mode="w") as entry_points_txt_file:
            entry_points_config.write(entry_points_txt_file)

    def modify_and_rebuild(self):


        # unpack wheel
        unpack_dir = tempfile.TemporaryDirectory()
        silent_unpack(path=str(self.wheel_dist_info.wheel_path), dest=unpack_dir.name)
        unpacked_wheel = list(Path(unpack_dir.name).iterdir())[0]

        # modify entry_points.txt in unpacked wheel
        self.modify_entry_points_txt(
            unpacked_wheel_path=unpacked_wheel,
            config_modifier=self.fix_console_script_names,
        )

        # repack wheel
        rebuild_dir = tempfile.TemporaryDirectory()
        silent_pack(
            directory=str(unpacked_wheel),
            dest_dir=str(rebuild_dir.name),
            build_number=None,
        )
        new_wheel_path = next(Path(rebuild_dir.name).glob("*.whl"), None)
        if not new_wheel_path:
            raise FileNotFoundError("Failed to create new wheel file")

        # overwrite old wheel with new
        shutil.move(str(new_wheel_path), str(self.wheel_dist_info.wheel_path))

        # clean up
        shutil.rmtree(rebuild_dir.name)


# if __name__ == "__main__":
#
#     my_wheel_path = (
#         (Path(__file__)).parent.parent.parent
#         / "numpy-2.2.2-cp310-cp310-manylinux_2_17_x86_64.manylinux2014_x86_64.whl"
#     )
#     my_wheel_dist_info = WheelDistInfo(wheel_path=my_wheel_path)
#     my_dist_entry_points_config = my_wheel_dist_info.get_entry_points_config()
#     my_console_script_names = my_wheel_dist_info.console_script_names
#     print(my_console_script_names)
#
#     wheel_modifier = WheelEntryPointsModifier(my_wheel_path)
#     wheel_modifier.modify_and_rebuild()
#     updated_console_script_names = my_wheel_dist_info.console_script_names
#     print(updated_console_script_names)



import entry_points_txt
import pytest
from pathlib import Path
from zipfile import ZipFile
import srepkg.error_handling.custom_exceptions as ce
import srepkg.utils.wheel_entry_point_extractor as we_pe
from shared_fixtures import sample_pkgs


@pytest.fixture
def testproj_wheel(sample_pkgs):
    return we_pe.WheelEntryPointExtractor(Path(sample_pkgs.testproj_whl))


class TestWheelEntryPointExtractor:

    def test_no_entry_pts_text(self, mocker, testproj_wheel):
        mocker.patch.object(ZipFile, "namelist", return_value=[])
        with pytest.raises(ce.NoEntryPtsTxtFile):
            testproj_wheel.get_entry_points()

    def test_multiple_entry_pts_text(self, mocker, testproj_wheel):
        mocker.patch.object(
            ZipFile,
            "namelist",
            return_value=["dir1/entry_points.txt", "dir2/entry_points.txt"],
        )
        with pytest.raises(ce.MultipleEntryPtsTxtFiles):
            testproj_wheel.get_entry_points()

    def test_no_console_scripts(self, mocker, testproj_wheel):
        mocker.patch.object(
            entry_points_txt, "load", return_value={"console_scripts": {}}
        )
        with pytest.raises(ce.NoConsoleScriptEntryPoints):
            testproj_wheel.get_entry_points()



from pathlib import Path
import srepkg.repackager as rep
from srepkg.srepkg import main
from shared_fixtures import sample_pkgs


def test_srepkg(mocker, sample_pkgs):
    mocker.patch.object(rep.Repackager, "repackage", return_value=None)

    main([sample_pkgs.wheel_inspect_whl])



import configparser
from pathlib import Path
from setuptools.command.install import install
from setuptools.command.develop import develop
from setuptools.command.egg_info import egg_info
import inner_pkg_installer.inner_pkg_installer as ipi


class InnerPkgCfgReader:
    def __init__(self, inner_pkg_cfg: Path):
        self._inner_pkg_cfg_file = inner_pkg_cfg
        self._inner_pkg_cfg = configparser.ConfigParser()
        self._inner_pkg_cfg.read(self._inner_pkg_cfg_file)

    @property
    def srepkg_name(self):
        return self._inner_pkg_cfg.get("metadata", "srepkg_name")

    @property
    def dist_dir(self):
        return self._inner_pkg_cfg.get("metadata", "dist_dir")

    @property
    def sdist_src(self):
        return self._inner_pkg_cfg.get("metadata", "sdist_src")


class CmdClassInstaller(ipi.InnerPkgInstaller):
    def __init__(self, srepkg_root: Path):
        install_cfg = InnerPkgCfgReader(srepkg_root / "inner_pkg_install.cfg")
        super().__init__(
            venv_path=srepkg_root / install_cfg.srepkg_name / "srepkg_venv",
            orig_pkg_dist=srepkg_root
            / install_cfg.dist_dir
            / install_cfg.sdist_src,
        )


class CustomInstallCommand(install):
    def run(self):

        install.run(self)
        CmdClassInstaller(
            srepkg_root=Path(__file__).parent.absolute()
        ).iso_install_inner_pkg()


class CustomDevelopCommand(develop):
    def run(self):

        develop.run(self)
        CmdClassInstaller(
            srepkg_root=Path(__file__).parent.absolute()
        ).iso_install_inner_pkg()


class CustomEggInfoCommand(egg_info):
    def run(self):

        egg_info.run(self)
        CmdClassInstaller(
            srepkg_root=Path(__file__).parent.absolute()
        ).iso_install_inner_pkg()


if __name__ == "__main__":
    CmdClassInstaller(
        srepkg_root=Path(__file__).parent.absolute()
    ).iso_install_inner_pkg()



"""
Used building a wheel distribution of re-packaged package.
"""

import setuptools
import zipfile
from pathlib import Path


class LocalOrigWheelExaminer:
    """
    Determines whether the original package wheel is pure python or
    platform-specific
    """
    def __init__(self):
        """
        Initializes the path of directory containing original package
        wheel. Location relative to Path(__file__) is known based on
        file structure design spec.
        """
        self.orig_dist_dir = Path(__file__).parent / "orig_dist"

    @property
    def orig_wheel_path(self) -> Path:
        """
        Absolute path of the original package wheel.
        """
        orig_dist_dir = Path(__file__).parent / "orig_dist"
        orig_dist_wheels = [
            path
            for path in orig_dist_dir.iterdir()
            if path.name.endswith(".whl")
        ]
        if len(orig_dist_wheels) == 0:
            raise FileNotFoundError("Original package wheel not found.")
        if len(orig_dist_wheels) > 1:
            raise FileExistsError(
                "More than one original package wheel found."
            )
        return orig_dist_wheels[0]

    @property
    def wheel_contents(self) -> list[str]:
        """
        List of paths of all contents in wheel, relative to wheel root.
        """
        with zipfile.ZipFile(str(self.orig_wheel_path), mode="r") as whl:
            contents = whl.namelist()
        return contents

    @property
    def dist_info_dir(self) -> str:
        """
        Path of wheel's .dist-info directory, relative to wheel root.
        """
        dist_info_dirs = {
            path.split("/", 1)[0]
            for path in self.wheel_contents
            if path.split("/", 1)[0].endswith(".dist-info")
        }

        if len(dist_info_dirs) == 0:
            raise ValueError("No dist-info directory")
        if len(dist_info_dirs) > 1:
            raise ValueError("Multiple dist-info directories found")
        return next(iter(dist_info_dirs))

    @property
    def wheel_info_path(self) -> str:
        """
        Path of the WHEEL metadata file relative to wheel root.
        """
        return f"{self.dist_info_dir}/WHEEL"

    @property
    def contains_wheel_info_file(self) -> bool:
        return self.wheel_info_path in self.wheel_contents

    @property
    def is_pure_python(self) -> bool | None:
        with zipfile.ZipFile(self.orig_wheel_path, mode="r") as whl:
            with whl.open(self.wheel_info_path) as f:
                for line in f.read().decode("utf-8").splitlines():
                    if line.startswith("Root-Is-Purelib:"):
                        return line.split(": ")[1].strip().lower() == "true"
        raise ValueError("Cannot determine if original package wheel is pure python")


class BinaryDistribution(setuptools.dist.Distribution):
    """Distribution which always forces a binary package with platform name"""

    def has_ext_modules(self):
        return True

orig_wheel_examiner = LocalOrigWheelExaminer()

if orig_wheel_examiner.is_pure_python:
    setuptools.setup()
else:
    setuptools.setup(distclass=BinaryDistribution)



import unittest.mock as mock
import srepkg.repackager as rep
import srepkg.repackager_interfaces as rep_int
import srepkg.service_builder as sb
import srepkg.srepkg_builder as s_bldr

from shared_fixtures import sample_pkgs


def test_repackager(tmp_path_factory, sample_pkgs):
    dist_out_dir = tmp_path_factory.mktemp("dist_out")
    construction_dir = tmp_path_factory.mktemp("construction_dir")

    srepkg_command = rep_int.SrepkgCommand(
        orig_pkg_ref=sample_pkgs.wheel_inspect_whl,
        dist_out_dir=str(dist_out_dir),
        construction_dir=str(construction_dir),
    )

    service_class_builder = sb.ServiceBuilder(srepkg_command)
    repackager = rep.Repackager(
        srepkg_command=srepkg_command,
        service_class_builder=service_class_builder,
    )

    with mock.patch.object(
        s_bldr.SrepkgBuilder, "build", return_value=None
    ) as mock_build:
        repackager.repackage()
        mock_build.assert_called_once_with()



# https://stackoverflow.com/a/24176022/263998

from contextlib import contextmanager
import os


@contextmanager
def dir_change_to(new_dir):
    prev_dir = os.getcwd()
    os.chdir(os.path.expanduser(new_dir))
    try:
        yield
    finally:
        os.chdir(prev_dir)



import argparse
import build
import sys
from pathlib import Path


class _DistBuilderArgParser:

    def __init__(self):
        self._parser = argparse.ArgumentParser()

    def _define_args(self):
        self._parser.add_argument("distribution", type=str)
        self._parser.add_argument("source_dir", type=str)
        self._parser.add_argument("output_directory", type=str)

    def get_args(self, *args):
        self._define_args()
        args_namespace = self._parser.parse_args(*args)
        return _DistBuilder(**vars(args_namespace))


class _DistBuilder:

    def __init__(self, distribution: str, source_dir: str, output_directory: str):
        self._distribution = distribution
        self._source_dir = source_dir
        self._output_directory = output_directory

    def build_dist(self) -> Path:
        dist_builder = build.ProjectBuilder(
            source_dir=self._source_dir, python_executable=sys.executable
        )

        dist_path_str = dist_builder.build(
            distribution=self._distribution,
            output_directory=self._output_directory,
        )

        return Path(dist_path_str)


def main(*args):
    dist_builder = _DistBuilderArgParser().get_args(*args)
    dist_path = dist_builder.build_dist()


if __name__ == "__main__":
    main()



import sys
from setuptools.command.bdist_wheel import bdist_wheel


class BdistWheelAbi3(bdist_wheel):
    def get_tag(self):
        python, abi, plat = super().get_tag()

        if python.startswith("cp"):
            major, minor = sys.version_info[:2]
            return f"cp{major}{minor}", "abi3", plat

        return python, abi, plat



import logging
import subprocess
import tempfile
from pathlib import Path
from typing import Dict, List, Callable


class SubprocessError(Exception):
    def __init__(
        self,
        sub_process: subprocess.CompletedProcess,
        msg="Error occurred when running subprocess",
    ):
        self._sub_process = sub_process
        self._msg = msg

    def __str__(self):
        return f"{str(self._sub_process)} -> {self._msg}"


class LoggedErrDetectingSubprocess:

    def __init__(
        self,
        cmd: List[str],
        gen_logger_name: str,
        std_out_logger_name: str,
        std_err_logger_name: str,
        cwd: Path = Path.cwd(),
        default_exception: Callable[
            [subprocess.CompletedProcess], Exception
        ] = SubprocessError,
        exception_table: Dict[
            int, Callable[[subprocess.CompletedProcess], Exception]
        ] = None,
    ):
        self._cmd = cmd
        self._gen_logger_name = gen_logger_name
        self._std_out_logger_name = std_out_logger_name
        self._std_out_buffer = tempfile.NamedTemporaryFile()
        self._std_err_logger_name = std_err_logger_name
        self._std_err_buffer = tempfile.NamedTemporaryFile()
        if exception_table is None:
            exception_table = {}
        self._cwd = cwd
        self._default_exception = default_exception
        self._exception_table = exception_table

    def run(self):
        sub_proc = subprocess.run(
            self._cmd,
            cwd=self._cwd,
            stdout=self._std_out_buffer,
            stderr=self._std_err_buffer,
            universal_newlines=True,
        )

        self._std_out_buffer.seek(0)
        for line in self._std_out_buffer:
            logging.getLogger(self._gen_logger_name).info(
                line.decode("utf-8").strip()
            )

        self._std_err_buffer.seek(0)
        if sub_proc.returncode == 0:
            for line in self._std_err_buffer:
                logging.getLogger(self._gen_logger_name).info(
                    line.decode("utf-8").strip()
                )
        else:
            for line in self._std_err_buffer:
                logging.getLogger(
                    f"{self._std_err_logger_name}.{self._gen_logger_name}"
                ).error(line.decode("utf-8").strip())

            if (
                self._exception_table
                and sub_proc.returncode in self._exception_table
            ):
                raise self._exception_table[sub_proc.returncode](sub_proc)
            else:
                raise self._default_exception(sub_proc)



import tempfile

import pkginfo
from dataclasses import dataclass, field
from packaging.utils import parse_wheel_filename
from pathlib import Path
from typing import List, NamedTuple, Union


@dataclass
class CSEntryPoint:
    command: str
    module: str
    attr: str

    # def __post_init__(self):
    #     if "-" in self.command:
    #         self.command = self.command.replace("-", "_")

    @property
    def as_string(self):
        return f"{self.command} = {self.module}:{self.attr}"


@dataclass
class PkgCSEntryPoints:
    cs_entry_pts: List[CSEntryPoint]

    @property
    def as_cfg_string(self):
        as_string_list = [cse.as_string for cse in self.cs_entry_pts]
        return "\n" + "\n".join(as_string_list)


@dataclass
class DistInfo:
    path: Path
    dist_obj: pkginfo.Distribution


class UniquePkg(NamedTuple):
    name: str
    version: str


@dataclass
class ConstructionDirSummary:
    pkg_name: str
    pkg_version: str
    srepkg_name: str
    srepkg_root: Path
    orig_pkg_dists: Path
    srepkg_inner: Path
    dists: List[DistInfo] = field(default_factory=lambda: [])
    entry_pts: PkgCSEntryPoints = field(
        default_factory=lambda: PkgCSEntryPoints(cs_entry_pts=[])
    )
    temp_dir_obj: tempfile.TemporaryDirectory = None

    @property
    def has_wheel(self):
        return any(
            [type(dist.dist_obj) == pkginfo.Wheel for dist in self.dists]
        )

    @property
    def wheel_path(self):
        if self.has_wheel:
            return [
                dist.path
                for dist in self.dists
                if type(dist.dist_obj) == pkginfo.Wheel
            ][0]

    @property
    def has_platform_indep_wheel(self):
        if not self.has_wheel:
            return False
        name, version, bld, tag = parse_wheel_filename(self.wheel_path.name)
        return list(tag)[0].platform == "any"

    @property
    def has_sdist(self):
        return any(
            [type(dist.dist_obj) == pkginfo.SDist for dist in self.dists]
        )

    @property
    def sdist_path(self):
        if self.has_sdist:
            return [
                dist.path
                for dist in self.dists
                if type(dist.dist_obj) == pkginfo.SDist
            ][0]

    @property
    def src_for_srepkg_wheel(self) -> Union[Path, None]:
        if self.has_wheel:
            return self.wheel_path
        if self.has_sdist:
            return self.sdist_path

    @property
    def src_for_srepkg_sdist(self) -> Union[Path, None]:
        if self.has_platform_indep_wheel:
            return self.wheel_path
        if self.has_sdist:
            return self.sdist_path



import pytest
import subprocess
import srepkg.error_handling.custom_exceptions as ce
import srepkg.logging_initializer as lgr
import srepkg.utils.logged_err_detecting_subprocess as leds
from shared_fixtures import app_logger
import srepkg.dist_builder as db


class SubprocessException(Exception):
    def __init__(
        self,
        sub_process: subprocess.CompletedProcess,
        msg="Raised test exception requiring CompletedProcess as arg",
    ):
        self._sub_process = sub_process
        self._msg = msg

    def __str__(self):
        return f"{str(self._sub_process)} -> {self._msg}"


class TestLoggedErrorDetectingSubprocess:

    @pytest.mark.parametrize(
        "command, exception",
        [
            (["not_a_command"], FileNotFoundError),
            (["git", "asdfas"], ce.BuildSubprocessError),
        ],
    )
    def test_bad_command(self, command, exception, app_logger):
        bad_sub_proc = leds.LoggedErrDetectingSubprocess(
            cmd=command,
            gen_logger_name=__name__,
            std_out_logger_name="std_out",
            std_err_logger_name="std_err",
            default_exception=exception,
        )

        with pytest.raises(exception):
            bad_sub_proc.run()



"""
Contains classes for generating appropriate ServiceBuilder based on a SrepkgCommand.
"""

import sys
from functools import singledispatch
from pathlib import Path
from typing import List, Type, Union, Dict, Callable

import srepkg.construction_dir as cdn
import srepkg.dist_provider as opr
import srepkg.error_handling.custom_exceptions as ce
import srepkg.error_handling.error_messages as em
import srepkg.orig_src_preparer as osp
import srepkg.orig_src_preparer_interfaces as osp_int
import srepkg.remote_pkg_retriever as rpr
import srepkg.srepkg_builder as sbn

import srepkg.repackager_data_structs as rep_ds
import srepkg.repackager_interfaces as rep_int

from srepkg.utils.pkg_type_identifier import PkgRefType, PkgRefIdentifier


@singledispatch
def create_construction_dir(
    construction_dir_command, srepkg_name_command: str = None
) -> cdn.ConstructionDir:
    raise NotImplementedError


@create_construction_dir.register(type(None))
def _(construction_dir_command, srepkg_name_command: str = None):
    return cdn.TempConstructionDir(srepkg_name_command=srepkg_name_command)


@create_construction_dir.register(str)
def _(construction_dir_command, srepkg_name_command: str = None):
    return cdn.CustomConstructionDir(
        construction_dir_command=Path(construction_dir_command),
        srepkg_name_command=srepkg_name_command,
    )


@create_construction_dir.register(Path)
def _(construction_dir_command, srepkg_name_command: str = None):
    return cdn.CustomConstructionDir(
        construction_dir_command=construction_dir_command,
        srepkg_name_command=srepkg_name_command,
    )


class RetrieverProviderDispatch:
    """
    Determines which implementation of DistProviderInterface is needed,
    and (if necessary) which implementation of RemotePkgRetrieverInterface is needed.
    """

    def __init__(
        self,
        pkg_ref_command: str,
        construction_dir: cdn.ConstructionDir,
        version_command: str = None,
        git_ref: str = None,
    ):
        """

        Args:
            pkg_ref_command: package identifier (e.g. PyPI name, GitHub repo)
            construction_dir: object with path and methods management methods for build dir
            version_command: PyPI version
            git_ref: git commit or tag
        """
        self._pkg_ref_command = pkg_ref_command
        self._construction_dir = construction_dir
        self._version_command = version_command
        self._git_ref = git_ref

    def _create_for_local_src_nongit(
        self,
    ) -> List[osp_int.DistProviderInterface]:
        provider = opr.DistProviderFromSrc(
            src_path=Path(self._pkg_ref_command),
            dest_path=self._construction_dir.orig_pkg_dists,
        )
        return [provider]

    def _create_for_local_dist(self) -> List[osp_int.DistProviderInterface]:
        provider = opr.DistCopyProvider(
            src_path=Path(self._pkg_ref_command),
            dest_path=self._construction_dir.orig_pkg_dists,
        )
        return [provider]

    def _create_for_git_repo(self):
        retriever = rpr.GithubPkgRetriever(pkg_ref=self._pkg_ref_command)
        # provider = opr.DistProviderFromSrc(
        #     src_path=retriever.copy_dest,
        #     dest_path=self._construction_dir.orig_pkg_dists)
        provider = opr.DistProviderFromGitRepo(
            src_path=retriever.copy_dest,
            dest_path=self._construction_dir.orig_pkg_dists,
            git_ref=self._git_ref,
            version_command=self._version_command,
        )
        return [retriever, provider]

    def _create_for_pypi(self):
        retriever = rpr.PyPIPkgRetriever(
            pkg_ref=self._pkg_ref_command,
            copy_dest=self._construction_dir.orig_pkg_dists,
            version_command=self._version_command,
        )
        return [retriever]

    @property
    def _dispatch_table(self) -> Dict[PkgRefType, Callable]:
        return {
            # PkgRefType.LOCAL_SRC_GIT: self._create_for_local_src_git,
            PkgRefType.LOCAL_SRC_NONGIT: self._create_for_local_src_nongit,
            PkgRefType.LOCAL_DIST: self._create_for_local_dist,
            PkgRefType.GIT_REPO: self._create_for_git_repo,
            PkgRefType.PYPI_PKG: self._create_for_pypi,
        }

    def create(self) -> List:
        """

        Returns:
            1 or 2 element List with instance of a DistProviderInterface
            and/or a RemotePkgRetrieverInterface
        """
        pkg_ref_type = PkgRefIdentifier(
            self._pkg_ref_command
        ).identify_for_osp_dispatch()

        if pkg_ref_type == PkgRefType.UNKNOWN:
            sys.exit(em.PkgIdentifierError.PkgNotFound.msg)

        if pkg_ref_type == PkgRefType.MULTIPLE_POSSIBLE:
            sys.exit(em.PkgIdentifierError.MultiplePotentialPackages.msg)

        if self._git_ref and not pkg_ref_type == PkgRefType.GIT_REPO:
            raise ce.UnusableGitCommitRef(self._git_ref)

        if self._version_command and not pkg_ref_type == PkgRefType.PYPI_PKG:
            raise ce.UnusableVersionArgument(self._version_command)

        return self._dispatch_table[pkg_ref_type]()


class OrigSrcPreparerBuilder:

    def __init__(
        self,
        construction_dir_command: Union[str, None],
        orig_pkg_ref_command: str,
        srepkg_name_command: str = None,
        version_command: str = None,
        git_ref_command: str = None,
    ):
        self._construction_dir_command = construction_dir_command
        self._orig_pkg_ref_command = orig_pkg_ref_command
        self._srepkg_name_command = srepkg_name_command
        self._version_command = version_command
        self._git_ref_command = git_ref_command
        self._construction_dir_dispatch = create_construction_dir

    def create(self):
        construction_dir = self._construction_dir_dispatch(
            self._construction_dir_command, self._srepkg_name_command
        )

        retriever_provider = RetrieverProviderDispatch(
            pkg_ref_command=self._orig_pkg_ref_command,
            construction_dir=construction_dir,
            version_command=self._version_command,
            git_ref=self._git_ref_command,
        ).create()

        return osp.OrigSrcPreparer(
            retriever_provider=retriever_provider, receiver=construction_dir
        )


class SrepkgBuilderBuilder:

    def __init__(
        self,
        output_dir_command: Union[str, None],
        construction_dir_summary: rep_ds.ConstructionDirSummary,
    ):
        self._construction_dir_summary = construction_dir_summary
        if output_dir_command is None:
            output_dir_command = str(Path.cwd())
        self._output_dir = output_dir_command

    @property
    def _completer_dispatch(
        self,
    ) -> Dict[Type[sbn.SrepkgCompleter], Union[Path, None]]:
        return {
            sbn.SrepkgWheelCompleter: self._construction_dir_summary.src_for_srepkg_wheel,
            sbn.SrepkgSdistCompleter: self._construction_dir_summary.src_for_srepkg_sdist,
        }

    def create(self):

        completers = []
        for constructor, src_path in self._completer_dispatch.items():
            if src_path is not None:
                completers.append(
                    constructor(
                        orig_pkg_summary=self._construction_dir_summary,
                        dist_out_dir=Path(self._output_dir),
                    )
                )

        srepkg_builder = sbn.SrepkgBuilder(
            construction_dir_summary=self._construction_dir_summary,
            srepkg_completers=completers,
            output_dir=(
                Path(self._output_dir) if self._output_dir else Path.cwd()
            ),
        )

        return srepkg_builder


class ServiceBuilder(rep_int.ServiceBuilderInterface):
    """
    Contains methods for generating instances of appropriate
    OrigSrcPreparerInterface and SrepkgBuilderInterface for a SrepkgCommand
    """

    def __init__(self, srepkg_command: rep_int.SrepkgCommand):
        self._srepkg_command = srepkg_command

    def create_orig_src_preparer(self) -> rep_int.OrigSrcPreparerInterface:
        osp_builder = OrigSrcPreparerBuilder(
            construction_dir_command=self._srepkg_command.construction_dir,
            orig_pkg_ref_command=self._srepkg_command.orig_pkg_ref,
            srepkg_name_command=self._srepkg_command.srepkg_name,
            git_ref_command=self._srepkg_command.git_ref,
            version_command=self._srepkg_command.pypi_version,
        )
        return osp_builder.create()

    def create_srepkg_builder(
        self, construction_dir_summary: rep_ds.ConstructionDirSummary
    ) -> rep_int.SrepkgBuilderInterface:
        srepkg_builder_builder = SrepkgBuilderBuilder(
            output_dir_command=self._srepkg_command.dist_out_dir,
            construction_dir_summary=construction_dir_summary,
        )
        return srepkg_builder_builder.create()



import pytest
import srepkg.error_handling.custom_exceptions as ce
import srepkg.repackager_interfaces as rep_int
import srepkg.service_builder as sb
from shared_fixtures import sample_pkgs
import srepkg.utils.dist_archive_file_tools as daft


class TestOrigSrcPreparer:

    @pytest.mark.parametrize(
        "pkg_ref, git_ref, version_command, expected_archive_types",
        [
            ("testproj", None, None, {daft.ArchiveDistType.WHEEL}),
            ("testproj_whl", None, None, {daft.ArchiveDistType.WHEEL}),
            (
                "testproj_targz",
                None,
                None,
                {daft.ArchiveDistType.SDIST, daft.ArchiveDistType.WHEEL},
            ),
            (
                "testproj_zip",
                None,
                None,
                {daft.ArchiveDistType.SDIST, daft.ArchiveDistType.WHEEL},
            ),
            # ("black_github", None, None, {daft.ArchiveDistType.WHEEL}),
            (
                "black_github",
                "767604e03f5e454ae5b5c268cd5831c672f46de8",
                None,
                {daft.ArchiveDistType.WHEEL},
            ),
            ("scrape_py_pi", None, "0.11.1", {daft.ArchiveDistType.WHEEL}),
        ],
    )
    def test_sources(
        self,
        pkg_ref,
        git_ref,
        version_command,
        expected_archive_types,
        sample_pkgs,
    ):
        cmd = rep_int.SrepkgCommand(
            orig_pkg_ref=getattr(sample_pkgs, pkg_ref),
            git_ref=git_ref,
            pypi_version=version_command,
        )
        src_preparer = sb.ServiceBuilder(cmd).create_orig_src_preparer()
        src_preparer.prepare()
        final_dist_types = {
            daft.ArchiveIdentifier().id_dist_type(item)
            for item in src_preparer._receiver._orig_pkg_dists_contents
        }
        assert expected_archive_types == final_dist_types

    def test_bad_commit_ref(self, sample_pkgs):
        cmd = rep_int.SrepkgCommand(
            orig_pkg_ref=sample_pkgs.black_github, git_ref="bad_commit_ref"
        )
        src_preparer = sb.ServiceBuilder(cmd).create_orig_src_preparer()
        with pytest.raises(ce.GitCheckoutError):
            src_preparer.prepare()



import pkginfo
import pytest
from pathlib import Path
import srepkg.error_handling.custom_exceptions as ce


class TestCustomExceptions:

    @pytest.mark.parametrize(
        "exception_name, exception_args",
        [
            (ce.MissingOrigPkgContent, ("dummy_path",)),
            (ce.UnsupportedCompressionType, ("dummy_file",)),
            (ce.MultiplePackagesPresent, ([],)),
            (ce.TargetDistTypeNotSupported, (pkginfo.Develop,)),
            (ce.NoSDistForWheelConstruction, (Path("dummy_path"),)),
            (ce.NoEntryPtsTxtFile, (Path("dummy_path"),)),
            (ce.MultipleEntryPtsTxtFiles, (Path("dummy_path"),)),
            (ce.NoConsoleScriptEntryPoints, (Path("dummy_path"),)),
            (ce.GitCheckoutError, ("dummy_commit_ref",)),
            (ce.UnusableGitCommitRef, ("dummy_commit_ref",)),
            (ce.UnusableVersionArgument, ("dummy_version_arg",)),
            (
                ce.PkgVersionWithCommitRef,
                ("dummy_commit_ref", "dummy_pkg_version"),
            ),
        ],
    )
    def test_exception_init_and_print(
        self, exception_name, exception_args, capsys
    ):
        cur_exception = exception_name(*exception_args)
        print(cur_exception)
        stdout = capsys.readouterr().out
        assert stdout.strip() == cur_exception.__str__().strip()



import abc
from dataclasses import dataclass
from typing import Union

import srepkg.repackager_data_structs as rep_ds


@dataclass
class SrepkgCommand:
    orig_pkg_ref: str
    srepkg_name: Union[str, None] = None
    construction_dir: Union[str, None] = None
    dist_out_dir: Union[str, None] = None
    git_ref: Union[str, None] = None
    pypi_version: Union[str, None] = None
    logfile_dir: Union[str, None] = None


class SrepkgCommandInterface(abc.ABC):

    @abc.abstractmethod
    def get_args(self) -> SrepkgCommand:
        pass


# class PkgDistProviderInterface(abc.ABC):
#
#     @abc.abstractmethod
#     def provide(self):
#         pass


class OrigSrcPreparerInterface(abc.ABC):

    @abc.abstractmethod
    def prepare(self):
        """
        Prepares original package for repackaging. Saves prepared
        files to disk.

        Returns:
            None
        """
        pass


class SrepkgBuilderInterface(abc.ABC):

    @abc.abstractmethod
    def build(self):
        """
        Builds package with isolation layer (saves to disk)
        Returns:
            None
        """
        pass


class ServiceBuilderInterface(abc.ABC):
    """
    Interface with one method for preparing original package files for
    repackaging and one method for performing repackaging.
    """

    @abc.abstractmethod
    def create_orig_src_preparer(self) -> OrigSrcPreparerInterface:
        pass

    @abc.abstractmethod
    def create_srepkg_builder(
        self, construction_dir_summary: rep_ds.ConstructionDirSummary
    ) -> SrepkgBuilderInterface:
        pass



import pytest
import shutil
from pathlib import Path

import srepkg.construction_dir as cdn
import srepkg.error_handling.custom_exceptions as ce
import srepkg.repackager_interfaces as rep_int
import srepkg.service_builder as sb
from shared_fixtures import sample_pkgs, tmp_construction_dir


class TestConstructionDirInit:

    @staticmethod
    def standard_init_tests(construction_dir: cdn.ConstructionDir):
        assert construction_dir.srepkg_root.exists()
        assert len(construction_dir.srepkg_root.name) == 32
        assert construction_dir.srepkg_root.parent == construction_dir._root
        assert construction_dir.srepkg_inner.exists()
        assert len(construction_dir.srepkg_inner.name) == 32
        assert (
            construction_dir.srepkg_inner.parent
            == construction_dir.srepkg_root
        )
        assert (
            construction_dir.supported_dist_types == cdn.DEFAULT_DIST_CLASSES
        )
        assert construction_dir._root_contents == [
            construction_dir.srepkg_root
        ]
        assert set(construction_dir._srepkg_root_contents) == {
            construction_dir.srepkg_inner,
            construction_dir.orig_pkg_dists,
        }
        assert construction_dir.srepkg_inner_contents == []
        assert construction_dir.pypi_version is None
        assert construction_dir.wheel_path is None

    def test_init_custom_construction_dir(self, tmp_path):
        construction_dir = cdn.CustomConstructionDir(
            construction_dir_command=tmp_path
        )
        self.standard_init_tests(construction_dir)
        assert construction_dir._root == tmp_path

    def test_init_temp_construction_dir(self):
        construction_dir = cdn.TempConstructionDir()
        self.standard_init_tests(construction_dir)
        assert construction_dir._temp_dir_obj is not None
        assert construction_dir._root == Path(
            construction_dir._temp_dir_obj.name
        )


class TestConstructionDirFinalize:

    @pytest.fixture(autouse=True)
    def _get_sample_pkgs(self, sample_pkgs):
        self._pkg_refs = sample_pkgs

    @pytest.mark.parametrize(
        "orig_pkg, srepkg_name, orig_pkg_name, num_dists",
        [
            ("testproj", None, "testproj", 1),
            ("testproj", "custom_name", "testproj", 1),
            ("testproj_targz", None, "testproj", 2),
        ],
    )
    def test_good_construction_dir_conditions(
        self, orig_pkg, srepkg_name, orig_pkg_name, num_dists
    ):
        srepkg_command = rep_int.SrepkgCommand(
            orig_pkg_ref=getattr(self._pkg_refs, orig_pkg),
            srepkg_name=srepkg_name,
        )
        service_builder = sb.ServiceBuilder(srepkg_command)
        osp = service_builder.create_orig_src_preparer()
        construction_dir_summary = osp.prepare()
        assert construction_dir_summary.pkg_name == orig_pkg_name
        assert len(construction_dir_summary.dists) == num_dists

    def test_get_dist_info_no_supported_dist_types(self):
        construction_dir = cdn.TempConstructionDir()
        construction_dir._supported_dist_types = []
        result = construction_dir._get_dist_info(self._pkg_refs.testproj)
        assert result is None

    def test_multiple_packages(self, sample_pkgs):
        service_builder = sb.ServiceBuilder(
            rep_int.SrepkgCommand(orig_pkg_ref=self._pkg_refs.testproj)
        )
        osp = service_builder.create_orig_src_preparer()
        construction_dir_summary = osp.prepare()
        shutil.copy2(
            src=sample_pkgs.wheel_inspect_whl,
            dst=construction_dir_summary.orig_pkg_dists,
        )

        with pytest.raises(ce.MultiplePackagesPresent):
            osp._receiver.finalize()

    def test_finalized_with_no_orig_pkg(self, tmp_construction_dir):
        with pytest.raises(ce.MissingOrigPkgContent):
            tmp_construction_dir.finalize()

    def test_sdist_to_wheel_converter_with_no_sdist(
        self, tmp_construction_dir
    ):
        converter = cdn.SdistToWheelConverter(tmp_construction_dir)
        with pytest.raises(ce.NoSDistForWheelConstruction):
            converter.build_wheel()


class TestConstructionDirSettle:

    def test_temp_cd_settle(self, tmp_construction_dir):
        tmp_construction_dir.settle()

    def test_custom_cd_settle(self, tmp_path):
        custom_construction_dir = cdn.CustomConstructionDir(tmp_path)
        custom_construction_dir.settle()


# This test has long runtime but may be good edge-case check
# def test_numpy_sdist_to_wheel():
#     numpy_sdist_path = Path(
#         "/Users/duane/dproj/srepkg/src/srepkg/test/package_test_cases/"
#         "numpy-1.23.1.tar.gz")
#
#     cur_command = rep_int.SrepkgCommand(
#         orig_pkg_ref=str(numpy_sdist_path),
#         construction_dir="/Users/duane/srepkg_pkgs"
#     )
#
#     src_preparer = sb.ServiceBuilder(cur_command).create_orig_src_preparer()
#     src_preparer._retriever.retrieve()
#     src_preparer._provider.provide()
#
#     reviewer = cdn.ConstructionDirReviewer(src_preparer._receiver)
#     existing_dists_summary = reviewer.get_existing_dists_summary()
#     converter = cdn.SdistToWheelConverter(
#         construction_dir=src_preparer._receiver,
#         construction_dir_summary=existing_dists_summary
#     )
#
#     converter.build_wheel()



import pytest
import unittest.mock as mock
import srepkg.error_handling.custom_exceptions as ce
import srepkg.service_builder
import srepkg.srepkg_builder as sbn
import srepkg.service_builder as sb
import srepkg.repackager_interfaces as rep_int
from shared_fixtures import (
    tmp_construction_dir,
    sample_pkgs,
    dummy_cdir_summary,
)


class TestConstructionDirDispatch:

    def test_none_create_arg(self):
        construction_dir = sb.create_construction_dir(None)
        assert type(construction_dir).__name__ == "TempConstructionDir"

    def test_string_create_arg(self, tmp_path):
        construction_dir = sb.create_construction_dir(str(tmp_path))
        assert type(construction_dir).__name__ == "CustomConstructionDir"

    def test_path_create_arg(self, tmp_path):
        construction_dir = sb.create_construction_dir(tmp_path)
        assert type(construction_dir).__name__ == "CustomConstructionDir"

    def test_invalid_construction_dir_arg(self):
        with pytest.raises(NotImplementedError):
            construction_dir = sb.create_construction_dir(1)


class TestRetrieverProviderDispatch:
    osp_conditions = [
        ("testproj", ["DistProviderFromSrc"]),
        ("testproj_targz", ["DistCopyProvider"]),
        ("testproj_zip", ["DistCopyProvider"]),
        ("black_py_pi", ["PyPIPkgRetriever"]),
        ("black_github", ["GithubPkgRetriever", "DistProviderFromSrc"]),
    ]

    @pytest.mark.parametrize(
        "pkg_ref_attr, retriever_provider_components", osp_conditions
    )
    def test_conditions(
        self,
        pkg_ref_attr,
        retriever_provider_components,
        tmp_construction_dir,
        sample_pkgs,
    ):
        pkg_ref_command = getattr(sample_pkgs, pkg_ref_attr)
        retriever_provider = sb.RetrieverProviderDispatch(
            pkg_ref_command=pkg_ref_command,
            construction_dir=tmp_construction_dir,
        ).create()
        assert [
            (type(item).__name__ for item in retriever_provider)
            == retriever_provider_components
        ]

    def test_github_commit_arg_with_non_repo_source(
        self, tmp_construction_dir, sample_pkgs
    ):

        with pytest.raises(ce.UnusableGitCommitRef):
            retriever_provider = sb.RetrieverProviderDispatch(
                pkg_ref_command=sample_pkgs.testproj,
                construction_dir=tmp_construction_dir,
                git_ref="dummy_git_ref",
            ).create()

    def test_version_arg_for_non_pypi_pkg_ref(
        self, tmp_construction_dir, sample_pkgs
    ):

        with pytest.raises(ce.UnusableVersionArgument):
            retriever_provider = sb.RetrieverProviderDispatch(
                pkg_ref_command=sample_pkgs.testproj,
                construction_dir=tmp_construction_dir,
                version_command="1.0.0",
            ).create()


class TestSrepkgBuilderBuilder:

    def test_no_completer_sources(self, dummy_cdir_summary):
        with mock.patch(
            "srepkg.service_builder.SrepkgBuilderBuilder."
            "_completer_dispatch",
            new_callable=mock.PropertyMock,
        ) as mock_completer_dispatch:
            mock_completer_dispatch.return_value = {
                sbn.SrepkgWheelCompleter: None,
                sbn.SrepkgSdistCompleter: None,
            }
            srepkg_builder = sb.SrepkgBuilderBuilder(
                output_dir_command=None,
                construction_dir_summary=dummy_cdir_summary,
            ).create()

        mock_completer_dispatch.assert_called_once_with()


service_bldr_conditions = [
    "testproj",
    # "numpy_whl",
    # "testproj_whl"
]


class TestServiceBuilder:

    @pytest.mark.parametrize("pkg_ref_attr", service_bldr_conditions)
    @mock.patch.object(srepkg.service_builder.OrigSrcPreparerBuilder, "create")
    def test_create_osp(self, mock_create, pkg_ref_attr, sample_pkgs):
        srepkg_command = rep_int.SrepkgCommand(
            orig_pkg_ref=getattr(sample_pkgs, pkg_ref_attr)
        )
        service_builder = sb.ServiceBuilder(srepkg_command)
        osp = service_builder.create_orig_src_preparer()
        mock_create.assert_called_with()

    @pytest.mark.parametrize("pkg_ref_attr", service_bldr_conditions)
    @mock.patch("srepkg.srepkg_builder.SrepkgWheelCompleter")
    @mock.patch("srepkg.srepkg_builder.SrepkgSdistCompleter")
    def test_create_srepkg_builder(
        self,
        MockSrepkgWheelComleter,
        MockSrepkgSdistCompleter,
        pkg_ref_attr,
        sample_pkgs,
    ):
        srepkg_command = rep_int.SrepkgCommand(
            orig_pkg_ref=getattr(sample_pkgs, pkg_ref_attr)
        )
        service_builder = sb.ServiceBuilder(srepkg_command)
        osp = service_builder.create_orig_src_preparer()
        orig_src_summary = osp.prepare()
        srepkg_builder = service_builder.create_srepkg_builder(
            orig_src_summary
        )
        assert MockSrepkgWheelComleter.called
        assert MockSrepkgSdistCompleter.called



import pytest
from unittest import mock
from packaging.utils import parse_wheel_filename
import srepkg.remote_pkg_retriever as rpr
from shared_fixtures import sample_pkgs, tmp_construction_dir


class TestRemotePackageRetriever:

    @pytest.mark.parametrize(
        "pkg_retriever, t_pkg_ref, num_whl_download, num_sdist_download",
        [
            ("PyPIPkgRetriever", "scrape_py_pi", 1, 0),
            ("PyPIPkgRetriever", "numpy_py_pi", 1, 1),
            ("GithubPkgRetriever", "howdoi_github", 0, 0),
        ],
    )
    def test_retriever(
        self,
        pkg_retriever,
        t_pkg_ref,
        num_whl_download,
        num_sdist_download,
        sample_pkgs,
        tmp_construction_dir,
    ):
        retriever_constructor = getattr(rpr, pkg_retriever)
        pkg_ref = getattr(sample_pkgs, t_pkg_ref)

        if pkg_retriever == "GithubPkgRetriever":
            constructor_args = {"pkg_ref": pkg_ref}
        else:
            constructor_args = {
                "pkg_ref": pkg_ref,
                "copy_dest": tmp_construction_dir.orig_pkg_dists,
            }

        retriever_constructor(**constructor_args).run()
        dists_in_cdir = list(tmp_construction_dir.orig_pkg_dists.iterdir())
        num_whl = len(
            [item for item in dists_in_cdir if item.suffix == ".whl"]
        )
        num_sdist = len(
            [item for item in dists_in_cdir if item.suffix == ".gz"]
        )
        assert num_whl == num_whl_download
        assert num_sdist == num_sdist_download


class TestPyPIPkgRetriever:

    def test_custom_version(self, tmp_path):
        retriever = rpr.PyPIPkgRetriever(
            pkg_ref="scrape", copy_dest=tmp_path, version_command="0.11.1"
        )
        retriever.run()
        copy_dest_contents = list(tmp_path.iterdir())
        downloaded_whl = [
            item for item in copy_dest_contents if item.suffix == ".whl"
        ]
        downloaded_whl_filename = downloaded_whl[0].name
        name, version, bld, tag = parse_wheel_filename(downloaded_whl_filename)
        assert str(version) == retriever._version_command

    def test_no_pi_whl_no_sdist(self, tmp_path):
        with mock.patch.object(
            rpr.PyPIPkgRetriever, "_has_sdist", new_callable=mock.PropertyMock
        ) as mock_has_dist:
            mock_has_dist.return_value = False
            retriever = rpr.PyPIPkgRetriever(
                pkg_ref="numpy", copy_dest=tmp_path
            )
            retriever.run()

    def test_only_sdist(self, tmp_path):
        retriever = rpr.PyPIPkgRetriever(pkg_ref="howdoi", copy_dest=tmp_path)
        retriever.run()



import configparser
import shutil
from pathlib import Path
import srepkg.repackager_data_structs as re_ds


class EntryPointsBuilder:
    """
    Creates console entry script files in the env from which srepkg
    was called (outside of venv where original package gets installed).
    """

    def __init__(
        self,
        orig_pkg_entry_pts: re_ds.PkgCSEntryPoints,
        entry_pt_template: Path,
        srepkg_entry_pt_dir: Path,
        srepkg_name: str,
        srepkg_config: configparser.ConfigParser,
        generic_entry_funct_name: str = "entry_funct",
    ):
        self._orig_pkg_entry_pts = orig_pkg_entry_pts
        self._entry_pt_template = entry_pt_template
        self._srepkg_entry_pt_dir = srepkg_entry_pt_dir
        self._srepkg_name = srepkg_name
        self._srepkg_config = srepkg_config
        self._generic_entry_funct_name = generic_entry_funct_name

    @property
    def _srepkg_entry_pts(self) -> re_ds.PkgCSEntryPoints:
        srepkg_cse_list = [
            re_ds.CSEntryPoint(
                command=orig_pkg_cse.command,
                module=".".join(
                    [
                        self._srepkg_name,
                        self._srepkg_entry_pt_dir.name,
                        orig_pkg_cse.command,
                    ]
                ),
                attr=self._generic_entry_funct_name,
            )
            for orig_pkg_cse in self._orig_pkg_entry_pts.cs_entry_pts
        ]
        return re_ds.PkgCSEntryPoints(srepkg_cse_list)

    def _write_entry_point_files(self):
        for cse in self._srepkg_entry_pts.cs_entry_pts:
            shutil.copy2(
                self._entry_pt_template,
                self._srepkg_entry_pt_dir / f"{cse.command}.py",
            )

        return self

    def _write_entry_point_init(self):
        import_statements = [
            f"import {self._srepkg_name}.srepkg_entry_points.{cse.command}"
            for cse in self._srepkg_entry_pts.cs_entry_pts
        ]

        with (self._srepkg_entry_pt_dir / "__init__.py").open(mode="w") as ei:
            for import_statement in import_statements:
                ei.write("".join([import_statement, "\n"]))
            ei.write("\n")

        return self

    def _update_srepkg_config(self):
        self._srepkg_config.set(
            "options.entry_points",
            "console_scripts",
            self._srepkg_entry_pts.as_cfg_string,
        )

        return self

    def build_entry_pts(self):
        """
        Creates console entry points in env from which srepkg was called.
        Returns:
            None
        """
        self._write_entry_point_files()
        self._write_entry_point_init()
        self._update_srepkg_config()



"""
Contains classes responsible for building sdist and/or wheel from the venv
(happens after original package has been installed in venv)
"""

import abc
import configparser
import logging
import shutil
import string
from dataclasses import dataclass
from pathlib import Path
import tarfile
import tempfile
from typing import Callable, List, NamedTuple
from zipfile import ZIP_DEFLATED, ZipFile

import inner_pkg_installer.inner_pkg_installer as ipi
import inner_pkg_installer.yaspin_updater as yu
import srepkg.cs_entry_pts as cse
import srepkg.dist_builder as db
import srepkg.repackager_data_structs as re_ds
import srepkg.repackager_interfaces as re_int


class TemplateWriteOp(NamedTuple):
    src_filename: str
    dest_filename: str
    replacement_map: dict


class SrepkgDistWriter(abc.ABC):
    """
    Interface with method for creating a distribution of a SRE package.
    """

    def __init__(
        self,
        orig_pkg_summary: re_ds.ConstructionDirSummary,
        dist_out_dir: Path,
    ):
        self._orig_pkg_summary = orig_pkg_summary
        self._dist_out_dir = dist_out_dir

    @abc.abstractmethod
    def write_dist(self) -> str:
        pass


class SrepkgSdistWriter(SrepkgDistWriter):
    """
    Creates a source distribution of SRE package.
    """

    @property
    def src_path(self) -> Path:
        return self._orig_pkg_summary.srepkg_root

    @property
    def exclude_paths(self) -> List[Path]:
        return [
            item
            for item in list((self.src_path / "orig_dist").iterdir())
            if item != self._orig_pkg_summary.src_for_srepkg_sdist
        ]

    @property
    def files_to_copy(self) -> List[Path]:
        return [
            path
            for path in list(self.src_path.rglob("*"))
            if path not in self.exclude_paths
        ]

    def write_to_zip(self, zip_name: str):
        with ZipFile(zip_name, "w", ZIP_DEFLATED) as zf:
            for file in self.files_to_copy:
                zf.write(file, file.relative_to(self.src_path.parent))

    def write_to_tar_gz(self, tar_gz_path: Path):
        with tarfile.open(tar_gz_path, "w:gz") as tar:
            for file in self.files_to_copy:
                tar.add(file, arcname=file.relative_to(self.src_path.parent))

    def write_dist(self):
        with yu.yaspin_log_updater(
            msg="Building srepkg sdist", logger=logging.getLogger(__name__)
        ):

            tar_filename = (
                f"{self._orig_pkg_summary.srepkg_name}-"
                f"{self._orig_pkg_summary.pkg_version}.tar.gz"
            )

            tar_sdist_path = self._dist_out_dir / tar_filename
            self.write_to_tar_gz(tar_gz_path=tar_sdist_path)

        logging.getLogger(f"std_out.{__name__}").info(
            f"\t{self._orig_pkg_summary.srepkg_name} sdist saved as: "
            f"{str(tar_sdist_path)}"
        )
        return str(tar_sdist_path)


class SrepkgWheelWriter(SrepkgDistWriter):
    """
    Creates a wheel distribution of SRE package.
    """

    def write_dist(self):
        with yu.yaspin_log_updater(
            msg="Building srepkg wheel", logger=logging.getLogger(__name__)
        ) as updater:
            wheel_path = db.DistBuilder(
                distribution="wheel",
                source_dir=self._orig_pkg_summary.srepkg_root,
                output_directory=self._dist_out_dir,
            ).build()

        logging.getLogger(f"std_out.{__name__}").info(
            f"\t{self._orig_pkg_summary.srepkg_name} wheel saved as: {wheel_path}"
        )

        return wheel_path


@dataclass
class CompleterProperties:
    components_dir: Path
    templates_dir: Path
    dist_writer_type: Callable[..., SrepkgDistWriter]


class SrepkgCompleter(abc.ABC):
    """
    Interface (with many concrete methods) for handling file copying and
    modification to prepare of dist creation, and creating distribution.
    """

    def __init__(
        self,
        orig_pkg_summary: re_ds.ConstructionDirSummary,
        dist_out_dir: Path,
        repkg_components: Path = Path(__file__).parent.absolute()
        / "repackaging_components",
    ):
        self._orig_pkg_summary = orig_pkg_summary
        self._dist_out_dir = dist_out_dir
        self._repkg_components = repkg_components

    @property
    @abc.abstractmethod
    def _props(self) -> CompleterProperties:
        pass

    @property
    def _dist_writer(self) -> SrepkgDistWriter:
        return self._props.dist_writer_type(
            orig_pkg_summary=self._orig_pkg_summary,
            dist_out_dir=self._dist_out_dir,
        )

    @property
    def _template_write_ops(self) -> List[TemplateWriteOp]:
        return [
            TemplateWriteOp(
                src_filename="MANIFEST.in.template",
                dest_filename="MANIFEST.in",
                replacement_map={
                    "srepkg_name": self._orig_pkg_summary.srepkg_name
                },
            )
        ]

    def _copy_ready_components(self):
        for item in self._props.components_dir.iterdir():
            if item.is_dir():
                shutil.copytree(
                    item, self._orig_pkg_summary.srepkg_root / item.name
                )
            else:
                shutil.copy2(item, self._orig_pkg_summary.srepkg_root)

    def _write_from_templates(self):
        for item in self._template_write_ops:
            with (self._props.templates_dir / item.src_filename).open(
                mode="r"
            ) as t:
                template_text = t.read()
            template = string.Template(template_text)
            result = template.substitute(item.replacement_map)
            with (
                self._orig_pkg_summary.srepkg_root / item.dest_filename
            ).open(mode="w") as f:
                f.write(result)

    @property
    def _srepkg_cfg_components(self) -> List[Path]:
        return [
            self._orig_pkg_summary.srepkg_root / "base_setup.cfg",
            self._props.templates_dir / "cmd_class.cfg",
        ]

    def _write_srepkg_setup_cfg(self):
        config = configparser.ConfigParser()
        config.read(src_path for src_path in self._srepkg_cfg_components)
        with (self._orig_pkg_summary.srepkg_root / "setup.cfg").open(
            mode="w"
        ) as cfg_file:
            config.write(cfg_file)

    def _restore_construction_dir_to(self, initial_contents: List[Path]):
        cur_dirs = [
            item
            for item in list(self._orig_pkg_summary.srepkg_root.rglob("*"))
            if item.is_dir()
        ]

        for item in cur_dirs:
            if item not in initial_contents:
                shutil.rmtree(item, ignore_errors=True)

        cur_files = [
            item
            for item in list(self._orig_pkg_summary.srepkg_root.rglob("*"))
            if not item.is_dir()
        ]

        for item in cur_files:
            if item not in initial_contents:
                item.unlink()

    @abc.abstractmethod
    def _extra_construction_tasks(self):
        pass

    def _write_dist(self):
        return self._dist_writer.write_dist()

    def build_and_cleanup(self) -> str:
        """
        Builds distribution and cleans up directory used for dist building.
        Returns: Path to the created distribution.
        """
        initial_contents = list(self._orig_pkg_summary.srepkg_root.rglob("*"))
        self._copy_ready_components()
        self._write_from_templates()
        self._write_srepkg_setup_cfg()
        self._extra_construction_tasks()
        dist_path = self._write_dist()
        self._restore_construction_dir_to(initial_contents)

        return dist_path


class SrepkgWheelCompleter(SrepkgCompleter):
    """
    Completes construction of a wheel distribution.
    """

    @property
    def _props(self) -> CompleterProperties:
        return CompleterProperties(
            components_dir=self._repkg_components
            / "wheel_completer_components",
            dist_writer_type=SrepkgWheelWriter,
            templates_dir=self._repkg_components / "wheel_completer_templates",
        )

    def _install_inner_pkg(self):
        ipi.InnerPkgInstaller(
            venv_path=self._orig_pkg_summary.srepkg_inner / "srepkg_venv",
            orig_pkg_dist=self._orig_pkg_summary.src_for_srepkg_wheel,
        ).iso_install_inner_pkg()

    def _extra_construction_tasks(self):
        self._install_inner_pkg()


class SrepkgSdistCompleter(SrepkgCompleter):
    """
    Completes construction of a sdist distribution.
    """

    @property
    def _props(self) -> CompleterProperties:
        return CompleterProperties(
            components_dir=self._repkg_components
            / "sdist_completer_components",
            dist_writer_type=SrepkgSdistWriter,
            templates_dir=self._repkg_components / "sdist_completer_templates",
        )

    def _build_ipi_cfg(self):
        metadata = {
            "srepkg_name": self._orig_pkg_summary.srepkg_name,
            "dist_dir": "orig_dist",
            "sdist_src": self._orig_pkg_summary.src_for_srepkg_sdist.name,
        }

        ipi_config = configparser.ConfigParser()
        ipi_config.add_section("metadata")

        for key, value in metadata.items():
            ipi_config.set("metadata", key, value)

        with (
            self._orig_pkg_summary.srepkg_root / "inner_pkg_install.cfg"
        ).open(mode="w") as ipi_cfg_file:
            ipi_config.write(ipi_cfg_file)

    def _extra_construction_tasks(self):
        self._build_ipi_cfg()


class SrepkgBuilder(re_int.SrepkgBuilderInterface):
    """
    Builds sdist and wheel distribution of SRE package and output file and entry-point
    info to terminal.
    """

    def __init__(
        self,
        construction_dir_summary: re_ds.ConstructionDirSummary,
        output_dir: Path,
        srepkg_completers: List[SrepkgCompleter] = None,
    ):
        if srepkg_completers is None:
            srepkg_completers = []
        self._srepkg_completers = srepkg_completers
        self._construction_dir_summary = construction_dir_summary
        self._output_dir = output_dir
        self._base_setup_cfg = configparser.ConfigParser()
        self._base_setup_cfg.read(
            Path(__file__).parent
            / "repackaging_components"
            / "base_templates"
            / "base_setup.cfg"
        )

    def _simple_construction_tasks(self):
        (
            self._construction_dir_summary.srepkg_inner / "srepkg_entry_points"
        ).mkdir()
        (self._construction_dir_summary.srepkg_inner / "__init__.py").touch()

        return self

    def _build_entry_points(self):
        cse.EntryPointsBuilder(
            orig_pkg_entry_pts=self._construction_dir_summary.entry_pts,
            entry_pt_template=Path(__file__).parent
            / "repackaging_components"
            / "base_templates"
            / "generic_entry.py",
            srepkg_entry_pt_dir=self._construction_dir_summary.srepkg_inner
            / "srepkg_entry_points",
            srepkg_name=self._construction_dir_summary.srepkg_inner.name,
            srepkg_config=self._base_setup_cfg,
            generic_entry_funct_name="entry_funct",
        ).build_entry_pts()

        return self

    def _write_srepkg_cfg_non_entry_data(self):
        metadata = {
            "name": self._construction_dir_summary.srepkg_name,
            "version": self._construction_dir_summary.pkg_version,
        }

        for key, value in metadata.items():
            self._base_setup_cfg.set("metadata", key, value)

        return self

    def _build_base_setup_cfg(self):
        with (
            self._construction_dir_summary.srepkg_root / "base_setup.cfg"
        ).open(mode="w") as cfg_file:
            self._base_setup_cfg.write(cfg_file)

        return self

    def _build_srepkg_base(self):
        self._simple_construction_tasks()
        self._build_entry_points()
        self._write_srepkg_cfg_non_entry_data()
        self._build_base_setup_cfg()

    def _complete_dists(self):
        dist_paths = []

        for completer in self._srepkg_completers:
            dist_paths.append(completer.build_and_cleanup())

        return dist_paths

    def build(self):
        self._build_srepkg_base()
        dist_paths = self._complete_dists()

        entry_pts_message = "\n".join(
            [
                f"\t• {entry_point.command}"
                for entry_point in self._construction_dir_summary.entry_pts.cs_entry_pts
            ]
        )

        qualifier = (
            "either of the following commands:"
            if len(self._srepkg_completers) == 2
            else "the command:"
        )

        commands = "\n".join(
            [f"\t• pip install {dist_path}" for dist_path in dist_paths]
        )

        logging.getLogger(f"std_out.{__name__}").info(
            f"\n{self._construction_dir_summary.srepkg_name} can be installed "
            f"using {qualifier}\n{commands}"
            f"\nUpon installation, {self._construction_dir_summary.srepkg_name} "
            f"will provide access to the following command line entry points: "
            f"\n{entry_pts_message}"
        )



"""
Contains classes for retrieving packages from remote locations.
"""

import functools
import logging
import requests
import tempfile
from packaging.tags import sys_tags, Tag
from packaging.utils import parse_wheel_filename
from pathlib import Path
from typing import Dict, Any, List

import inner_pkg_installer.yaspin_updater as yu
import srepkg.error_handling.custom_exceptions as ce
import srepkg.orig_src_preparer_interfaces as osp_int
import srepkg.utils.logged_err_detecting_subprocess as leds


class PyPIPkgRetriever(osp_int.RemotePkgRetrieverInterface):
    """
    Retrieves packages from Python Package Index.
    """
    _pypi_api_base = "https://pypi.org/pypi/{}/json"

    def __init__(
        self, pkg_ref: str, copy_dest: Path, version_command: str = None
    ):
        self._pkg_ref = pkg_ref
        self._copy_dest = copy_dest
        self._version_command = version_command

    @property
    def _pkg_info_url(self) -> str:
        return self._pypi_api_base.format(self._pkg_ref)

    @property
    @functools.lru_cache(maxsize=128, typed=False)
    def _pkg_metadata(self) -> Dict[str, Any]:
        return requests.get(self._pkg_info_url).json()

    @property
    def _version_url_info(self) -> List[Dict[str, Any]]:
        if self._version_command:
            return self._pkg_metadata["releases"][self._version_command]
        else:
            return self._pkg_metadata["urls"]

    @property
    def _sdists(self) -> List[Dict[str, Any]]:
        return [
            dist
            for dist in self._version_url_info
            if dist["packagetype"] == "sdist"
        ]

    @property
    def _has_sdist(self) -> bool:
        return len(self._sdists) > 0

    @property
    def _wheels(self) -> List[Dict[str, Any]]:
        return [
            dist
            for dist in self._version_url_info
            if dist["packagetype"] == "bdist_wheel"
        ]

    @staticmethod
    def _get_tag(dist_entry: dict) -> Tag:
        name, version, bld, tag = parse_wheel_filename(dist_entry["filename"])
        return list(tag)[0]

    def _is_platform_indep_wheel(self, dist_entry: dict) -> bool:
        # Should we ignore abi when doing this check???
        return self._get_tag(dist_entry).platform == "any"

    @property
    def _platform_indep_wheels(self) -> List[Dict[str, Any]]:
        return [
            dist
            for dist in self._wheels
            if self._is_platform_indep_wheel(dist)
        ]

    @property
    def _has_platform_indep_wheel(self) -> bool:
        return len(self._platform_indep_wheels) > 0

    def _is_platform_specific_wheel(self, dist_entry: dict) -> bool:
        tag = self._get_tag(dist_entry)
        return tag.platform != "any"

    @property
    def _platform_specific_wheels(self) -> List[Dict[str, Any]]:
        return [
            dist
            for dist in self._wheels
            if self._is_platform_specific_wheel(dist)
        ]

    @property
    def _platform_specific_wheels_for_cur_sys(self) -> List[Dict[str, Any]]:
        return [
            dist
            for dist in self._platform_specific_wheels
            if self._get_tag(dist) in list(sys_tags())
        ]

    @property
    def _has_platform_specific_wheel_for_cur_sys(self) -> bool:
        return len(self._platform_specific_wheels_for_cur_sys) > 0

    @property
    def _dists_to_download(self) -> List[Dict[str, Any]]:
        dists_to_download = []
        if self._has_platform_indep_wheel:
            dists_to_download.append(self._platform_indep_wheels[0])
            return dists_to_download

        if self._has_sdist:
            dists_to_download.append(self._sdists[0])
        if self._has_platform_specific_wheel_for_cur_sys:
            dists_to_download.append(
                self._platform_specific_wheels_for_cur_sys[0]
            )

        return dists_to_download

    def _download(self, dist: dict) -> None:
        response = requests.get(dist["url"])
        with (self._copy_dest / dist["filename"]).open(mode="wb") as dist_file:
            dist_file.write(response.content)

    def run(self) -> None:

        with yu.yaspin_log_updater(
            msg=f"Retrieving {self._pkg_ref} from Python Packaging Index",
            logger=logging.getLogger(__name__),
        ) as updater:

            for dist in self._dists_to_download:
                self._download(dist)

        downloaded_files = "\n".join(
            [f"\t• {dist['filename']}" for dist in self._dists_to_download]
        )
        post_msg = f"\tDownloaded files:\n{downloaded_files}"
        logging.getLogger(f"std_out.{__name__}").info(post_msg)


class GithubPkgRetriever(osp_int.RemotePkgRetrieverInterface):
    """
    Retrieves packages from Github.
    """

    def __init__(self, pkg_ref: str):
        self._pkg_ref = pkg_ref
        self._temp_dir_obj = tempfile.TemporaryDirectory()

    @property
    def copy_dest(self):
        return Path(self._temp_dir_obj.name)

    def run(self):
        with yu.yaspin_log_updater(
            msg=f"Cloning {self._pkg_ref} into temporary directory",
            logger=logging.getLogger(__name__),
        ) as updater:
            leds.LoggedErrDetectingSubprocess(
                cmd=["git", "clone", self._pkg_ref, self.copy_dest],
                gen_logger_name=__name__,
                std_out_logger_name="std_out",
                std_err_logger_name="std_err",
                default_exception=ce.GitCheckoutError,
            ).run()



"""
Contains classes that handle venv creation and installation of original\
package in venv
"""

import configparser
import logging
import re
import subprocess
import sys
import tempfile
import venv
from contextlib import contextmanager
from datetime import datetime
from types import SimpleNamespace
from pathlib import Path
from typing import Dict
from unittest.mock import Mock

# try:
#     from .yaspin_status import yaspin_logging as show_status
# except(ModuleNotFoundError, ImportError):
#     from .simple_status import simple_status as show_status

try:
    from .yaspin_updater import yaspin_log_updater as status_display
except (ModuleNotFoundError, ImportError):

    @contextmanager
    def status_display(msg, logger: logging.Logger):
        logger.info(msg)
        yield


class IPILogging:
    """
    Takes care of logging during venv creation and installation of original
    package in venv.
    """

    @staticmethod
    def confirm_setup():

        custom_logger_names = [
            logging.getLogger(ref).name
            for ref in logging.root.manager.loggerDict
        ]

        console_logger_info = {
            "std_err": (logging.DEBUG, sys.stderr),
            "std_out": (logging.DEBUG, sys.stdout),
        }

        if not all(
            [key in custom_logger_names for key in console_logger_info]
        ):

            log_dir = tempfile.TemporaryDirectory()
            filename = (
                f"srepkg_log_"
                f"{datetime.now().strftime('%Y-%m-%d_%H_%M_%S_%f')}.log"
            )
            logging.basicConfig(
                level=logging.DEBUG,
                format="%(asctime)s:%(levelname)s:%(name)s:%(message)s",
                filename=f"{str(log_dir.name)}/{filename}",
                filemode="w",
            )

            for logger_name, handler_info in console_logger_info.items():
                if logger_name not in custom_logger_names:
                    logger = logging.getLogger(logger_name)
                    handler = logging.StreamHandler(stream=handler_info[1])
                    handler.setLevel(level=handler_info[0])
                    logger.addHandler(handler)


class SitePackagesInspector:
    """
    Gets versions of packages installed in venv.
    """

    def __init__(self, site_pkgs: Path):
        self._site_pkgs = site_pkgs

    @staticmethod
    def _get_val_from_line(data_key: str, line: str):
        data_regex = r"^" + re.escape(data_key) + r": [\d.a-zA-Z-_]+\s*$"
        if re.search(data_regex, line, re.IGNORECASE):
            return line[(len(data_key) + 2) :].strip()

    def _get_val_from_file(self, data_key: str, metadata_file: Path):
        with metadata_file.open(mode="r") as f:
            while True:
                line = f.readline()
                if not line:
                    break
                value = self._get_val_from_line(data_key, line)
                if value:
                    return value

    def _get_metadata_val_from_dist_info_dir(
        self, data_key: str, dist_info_dir: Path
    ):
        for item in dist_info_dir.iterdir():
            if item.name == "METADATA":
                return self._get_val_from_file(
                    data_key=data_key, metadata_file=item
                )

    def get_pkg_version(self, pkg_name: str):
        for item in self._site_pkgs.iterdir():
            if item.suffix == ".dist-info":
                found_pkg_name = self._get_metadata_val_from_dist_info_dir(
                    data_key="Name", dist_info_dir=item
                )
                if found_pkg_name == pkg_name:
                    return self._get_metadata_val_from_dist_info_dir(
                        data_key="Version", dist_info_dir=item
                    )


class PyVersion:

    def __init__(self, version_str: str):
        self._version_str = version_str

    @property
    def major(self):
        major_pattern = r"((?<=^)[\d]{1,2}(?=\.))"
        return int(re.findall(major_pattern, self._version_str)[0])

    @property
    def minor(self):
        minor_pattern = r"((?<=\.)[\d]{1,2}(?=\.))"
        return int(re.findall(minor_pattern, self._version_str)[0])

    @property
    def micro(self):
        micro_pattern = r"((?<=\.)[\d]{1,2}$)"
        return int(re.findall(micro_pattern, self._version_str)[0])


class CustomVenvBuilder(venv.EnvBuilder):
    """
    Sublcass of standard library EnvBuilder with extra methods used to
    install and/or upgrade pip, setuptools, and wheel in the venv.
    """
    def __init__(self):
        super().__init__(with_pip=True)
        self._context = SimpleNamespace()
        self._site_pkgs = None
        self._version_info = sys.version_info

    @property
    def context(self):
        return self._context

    @property
    def version_info(self):
        return self._version_info

    @property
    def site_pkgs(self):
        return self._site_pkgs

    def post_setup(self, context) -> None:
        subprocess.call(
            [
                context.env_exe,
                "-m",
                "pip",
                "install",
                "--upgrade",
                "pip",
                "--quiet",
            ]
        )
        subprocess.call(
            [
                context.env_exe,
                "-m",
                "pip",
                "install",
                "--upgrade",
                "setuptools",
                "--quiet",
            ]
        )
        subprocess.call(
            [
                context.env_exe,
                "-m",
                "pip",
                "install",
                "--upgrade",
                "wheel",
                "--quiet",
            ]
        )

        # self._site_pkgs = Path(
        #     f"{context.env_dir}/lib/python{sys.version_info.major}."
        #     f"{sys.version_info.minor}/site-packages")
        self._context = context


class VenvManager:
    """
    Provides methods for obtaining info about packages in venv and, most
    importantly, modifying shebangs of console script entry point files so
    they are accessible from environment srepkg was run from (outside of venv).
    """
    def __init__(self, context):
        self._context = context
        self._pyvenv_cfg = configparser.ConfigParser()
        with self._cfg_path.open(mode="r") as cfg_file:
            self._pyvenv_cfg.read_string("[pyvenv_cfg]\n" + cfg_file.read())
        self._console_scripts = []

    @property
    def _env_dir(self) -> Path:
        return Path(self._context.env_dir)

    @property
    def _env_exe(self):
        return Path(self._context.env_exe)

    @property
    def _cfg_path(self):
        return Path(self._context.cfg_path)

    @property
    def _version(self):
        return PyVersion(self._pyvenv_cfg.get("pyvenv_cfg", "version"))

    @property
    def _site_packages(self):

        return (
            self._env_dir / f"lib/python{str(self._version.major)}."
            f"{str(self._version.minor)}" / "site-packages"
        )

    @property
    def pypa_pkg_versions(self) -> Dict[str, str]:
        pypa_pkgs = ["pip", "setuptools", "wheel"]
        site_pkgs_inspector = SitePackagesInspector(self._site_packages)
        return {
            pkg: site_pkgs_inspector.get_pkg_version(pkg) for pkg in pypa_pkgs
        }

    @property
    def _relative_shebang(self):
        return '#!/bin/sh\n"exec" "`dirname $0`/python" "$0" "$@"\n'

    def pip_install(self, *args):
        subprocess.call([self._env_exe, "-m", "pip", "install"] + [*args])

        return self

    def _get_console_scripts(self, info_file: Path):
        config = configparser.ConfigParser()
        config.read(info_file)
        if "console_scripts" in config.sections():
            self._console_scripts += dict(
                config.items("console_scripts")
            ).keys()

    def _gather_all_console_scripts(self):
        self._console_scripts = []
        for path in self._site_packages.iterdir():
            if path.is_dir() and (
                path.name.endswith(".dist-info")
                or path.name.endswith(".egg-info")
            ):
                if (path / "entry_points.txt").exists():
                    self._get_console_scripts(path / "entry_points.txt")

    def _update_cs_shebang(self, cs_path: Path):
        """
        Modifies shebang of a console script file so it can be called from outside
        of venv (in the environment where srepkg was run from)

        Args:
            cs_path: Path of a console script (entry point).

        Returns:
            None
        """
        if cs_path.exists():
            with cs_path.open(mode="r+") as f:
                first_line = f.readline()
                second_line = f.readline()
                if first_line + second_line != self._relative_shebang:
                    remaining_lines = f.read()
                    f.seek(0)
                    f.write(
                        self._relative_shebang + second_line + remaining_lines
                    )
                    f.truncate()

    def _update_all_cs_shebangs(self):
        """
        Modifies shebang of all console scripts except any pip entry point script
        that does not have dist info.

        Returns:
            None
        """

        for console_script in self._console_scripts:
            self._update_cs_shebang(self._env_dir / "bin" / console_script)

    def _update_rogue_pip_cs_shebang(self):
        """
        Updates shebang of any pip entrypoint script that does not have dist info.
        Returns:

        """

        pip_cs_without_dist_info = (
            self._env_dir
            / "bin"
            / f"pip{str(self._version.major)}.{str(self._version.minor)}"
        )
        if pip_cs_without_dist_info.exists():
            self._update_cs_shebang(pip_cs_without_dist_info)

    def rewire_shebangs(self):
        """
        Modifies shebang of each console script in venv so it can be called
        form environment where srepkg was run from (outside of venv).
        Returns:
            None
        """
        self._gather_all_console_scripts()
        self._update_all_cs_shebangs()
        self._update_rogue_pip_cs_shebang()


class InnerPkgCfgReader:
    """
    Gets metadate from the original package.
    """
    def __init__(self, inner_pkg_cfg: Path):
        self._inner_pkg_cfg_file = inner_pkg_cfg
        self._inner_pkg_cfg = configparser.ConfigParser()
        self._inner_pkg_cfg.read(self._inner_pkg_cfg_file)

    @property
    def srepkg_name(self):
        return self._inner_pkg_cfg.get("metadata", "srepkg_name")

    @property
    def dist_dir(self):
        return self._inner_pkg_cfg.get("metadata", "dist_dir")

    @property
    def sdist_src(self):
        return self._inner_pkg_cfg.get("metadata", "sdist_src")


class InnerPkgInstaller:
    """
    Installs original package in venv.
    """
    def __init__(self, venv_path: Path, orig_pkg_dist: Path):
        self._venv_path = venv_path
        self._orig_pkg_dist = orig_pkg_dist
        self._venv_manager = None

    # @show_status(
    #     base_logger=logging.getLogger(__name__),
    #     std_out_logger=logging.getLogger(f"std_out.{__name__}"))
    def build_venv(
        self,
        # status_msg_kwarg
    ):
        env_builder = CustomVenvBuilder()
        env_builder.create(self._venv_path)

        return env_builder.context

    # @show_status(
    #     base_logger=logging.getLogger(__name__),
    #     std_out_logger=logging.getLogger(f"std_out.{__name__}"))
    def install_orig_pkg_in_venv(
        self,
        # status_msg_kwarg
    ):
        self._venv_manager.pip_install(
            self._orig_pkg_dist, "--quiet"
        ).rewire_shebangs()

    def show_post_venv_creation_summary(self):
        pkg_versions = "\n".join(
            [
                f"\t• {pkg}=={version}"
                for pkg, version in self._venv_manager.pypa_pkg_versions.items()
            ]
        )
        msg = (
            f"\tVirtual env created with the following pypa "
            f"packages installed:\n{pkg_versions}"
        )
        logging.getLogger(f"std_out.{__name__}").info(msg)

    def iso_install_inner_pkg(self):
        IPILogging.confirm_setup()

        with status_display(
            msg="Creating virtual env", logger=logging.getLogger(__name__)
        ) as updater:
            # spinner.text = "Creating virtual env"
            venv_context = self.build_venv()
            # spinner.ok("✔")

        self._venv_manager = VenvManager(context=venv_context)
        self.show_post_venv_creation_summary()

        with status_display(
            msg=f"Installing {self._orig_pkg_dist.name} in virtual env",
            logger=logging.getLogger(__name__),
        ) as updater:
            # spinner.text = f"Installing {self._orig_pkg_dist.name} in virtual env"
            self.install_orig_pkg_in_venv()



import abc
import logging
import pkginfo
import tempfile
import uuid
from pathlib import Path
from typing import List

from yaspin import yaspin

import srepkg.dist_builder as db
import srepkg.error_handling.custom_exceptions as ce
import srepkg.utils.dist_archive_file_tools as cft
import srepkg.orig_src_preparer_interfaces as osp_int
import srepkg.repackager_data_structs as rp_ds
import srepkg.wheel_modifier as wm
import srepkg.utils.wheel_entry_point_extractor as we_pe

from inner_pkg_installer import yaspin_updater as yu

DEFAULT_DIST_CLASSES = (pkginfo.SDist, pkginfo.Wheel)
DEFAULT_SREPKG_SUFFIX = "srepkg"


class ConstructionDir(osp_int.ManageableConstructionDir):
    """
    Class for managing directory where SRE package is built.
    """

    def __init__(
        self, construction_dir_command: Path, srepkg_name_command: str = None
    ):
        self._root = construction_dir_command
        self._srepkg_root = construction_dir_command / uuid.uuid4().hex
        self._srepkg_inner = self._srepkg_root / uuid.uuid4().hex
        self._srepkg_root.mkdir()
        self._srepkg_inner.mkdir()
        (self._srepkg_root / "orig_dist").mkdir()
        self._custom_srepkg_name = srepkg_name_command
        self._supported_dist_types = DEFAULT_DIST_CLASSES
        self._srepkg_name = None
        self._summary = None

    @property
    def _root_contents(self):
        return list(self._root.iterdir())

    @property
    def srepkg_root(self):
        return self._srepkg_root

    @property
    def _srepkg_root_contents(self):
        return list(self._srepkg_root.iterdir())

    @property
    def orig_pkg_dists(self) -> Path:
        return self._srepkg_root / "orig_dist"

    @property
    def _orig_pkg_dists_contents(self) -> List[Path]:
        return list(self.orig_pkg_dists.iterdir())

    def _get_dist_info(self, dist_path: Path):
        for dist_class in self.supported_dist_types:
            try:
                dist_obj = dist_class(dist_path)
                return rp_ds.DistInfo(path=dist_path, dist_obj=dist_obj)
            except ValueError:
                pass

    @property
    def dists(self):
        return [
            self._get_dist_info(entry)
            for entry in self._orig_pkg_dists_contents
        ]

    @property
    def _unique_orig_pkgs(self):
        unique_pkgs = {
            rp_ds.UniquePkg(
                # DistInfo changes any "_" to "-" in pkg name. Undo that.
                name=dist.dist_obj.name.replace("-", "_"),
                version=dist.dist_obj.version,
            )
            for dist in self.dists
        }
        if len(unique_pkgs) > 1:
            raise ce.MultiplePackagesPresent(self.dists)
        return unique_pkgs

    @property
    def orig_pkg_name(self):
        if self._unique_orig_pkgs:
            return list(self._unique_orig_pkgs)[0].name

    @property
    def pypi_version(self):
        if self._unique_orig_pkgs:
            return list(self._unique_orig_pkgs)[0].version

    @property
    def has_wheel(self):
        return any(
            [type(dist.dist_obj) == pkginfo.Wheel for dist in self.dists]
        )

    @property
    def wheel_path(self):
        if self.has_wheel:
            return [
                dist.path
                for dist in self.dists
                if type(dist.dist_obj) == pkginfo.Wheel
            ][0]

    @property
    def has_sdist(self):
        return any(
            [type(dist.dist_obj) == pkginfo.SDist for dist in self.dists]
        )

    # @property
    # def srepkg_name(self) -> str:
    #     return self._srepkg_name

    @property
    def srepkg_inner(self):
        return self._srepkg_inner

    @property
    def srepkg_inner_contents(self):
        return list(self._srepkg_inner.iterdir())

    @property
    def supported_dist_types(self):
        return self._supported_dist_types

    def _rename_sub_dirs(self, srepkg_root_new: str, srepkg_inner_new: str):

        self._srepkg_inner.replace(
            self._srepkg_inner.parent.absolute() / srepkg_inner_new
        )
        self._srepkg_root.replace(
            self._srepkg_root.parent.absolute() / srepkg_root_new
        )

        self._srepkg_root = (
            self._srepkg_root.parent.absolute() / srepkg_root_new
        )
        self._srepkg_inner = self._srepkg_root / srepkg_inner_new

    def _update_srepkg_and_dir_names(self, discovered_pkg_name: str):
        if self._custom_srepkg_name:
            srepkg_name = self._custom_srepkg_name
        else:
            srepkg_name = f"{discovered_pkg_name}{DEFAULT_SREPKG_SUFFIX}"

        self._rename_sub_dirs(
            srepkg_root_new=f"{discovered_pkg_name}_as_{srepkg_name}",
            srepkg_inner_new=srepkg_name,
        )

        self._srepkg_name = srepkg_name

    def _ensure_valid_console_script_names(self):
        wheel_dist_info = wm.WheelDistInfo(wheel_path=self.wheel_path)
        if wheel_dist_info.has_console_script_name_with_dash:
            wm.WheelEntryPointsModifier(
                wheel_path=self.wheel_path
            ).modify_and_rebuild()

    def _extract_cs_entry_pts_from_wheel(self):
        return we_pe.WheelEntryPointExtractor(
            self.wheel_path
        ).get_entry_points()

    def _ensure_have_wheel(self):
        if not self.has_sdist and not self.has_wheel:
            raise ce.MissingOrigPkgContent(str(self.orig_pkg_dists))
        if not self.has_wheel and self.has_sdist:
            SdistToWheelConverter(self).build_wheel()

    def _set_summary(self):

        self._summary = rp_ds.ConstructionDirSummary(
            pkg_name=self.orig_pkg_name,
            pkg_version=self.pypi_version,
            srepkg_name=self._srepkg_name,
            srepkg_root=self._srepkg_root,
            orig_pkg_dists=self.orig_pkg_dists,
            srepkg_inner=self._srepkg_inner,
            dists=self.dists,
            entry_pts=self._extract_cs_entry_pts_from_wheel(),
        )

    def finalize(self):
        self._ensure_have_wheel()
        self._update_srepkg_and_dir_names(
            discovered_pkg_name=self.orig_pkg_name
        )

        self._ensure_valid_console_script_names()
        self._set_summary()
        return self._summary

    @abc.abstractmethod
    def settle(self):
        pass


class CustomConstructionDir(ConstructionDir):
    """
    Sublcass of ConstructionDir used when SrepkgCommand specifies a build
    location.
    """

    def __init__(
        self, construction_dir_command: Path, srepkg_name_command: str = None
    ):
        super().__init__(construction_dir_command, srepkg_name_command)

    def settle(self):
        print(
            f"An uncompressed copy of {self._srepkg_inner.name} has been saved "
            f"in {str(self._srepkg_root)}"
        )


class TempConstructionDir(ConstructionDir):
    """
    Sublcass of ConstructionDir used when SrepkgCommand does not specify a
    build location (and temp dir is used).
    """

    def __init__(self, srepkg_name_command: str = None):
        self._temp_dir_obj = tempfile.TemporaryDirectory()
        super().__init__(
            construction_dir_command=Path(self._temp_dir_obj.name),
            srepkg_name_command=srepkg_name_command,
        )

    def _set_summary(self):
        super()._set_summary()
        self._summary._temp_dir_obj = self._temp_dir_obj

    def settle(self):
        self._temp_dir_obj.cleanup()


class SdistToWheelConverter:
    """
    Converts a SDist to Wheel. Use this when original package is only in
    SDist form, so we can install with wheel into venv.
    """

    def __init__(self, construction_dir: ConstructionDir):
        self._construction_dir = construction_dir
        self._compressed_file_extractor = cft.CompressedFileExtractor()

    @property
    def _unpacked_src_dir_name(self):
        pkg_name = self._construction_dir.orig_pkg_name
        pkg_version = self._construction_dir.pypi_version
        return f"{pkg_name}-{pkg_version}"

    def _get_build_from_dist(self):
        try:
            build_from_dist = next(
                dist
                for dist in self._construction_dir.dists
                if isinstance(dist.dist_obj, pkginfo.sdist.SDist)
            )
        except StopIteration:
            raise ce.NoSDistForWheelConstruction(
                self._construction_dir.srepkg_root
            )

        return build_from_dist

    def build_wheel(self):

        build_from_dist = self._get_build_from_dist()

        with yu.yaspin_log_updater(
            msg=f"Converting {build_from_dist.path.name} to a wheel",
            logger=logging.getLogger(__name__),
        ) as updater:
            temp_unpack_dir_obj = tempfile.TemporaryDirectory()
            unpack_root = Path(temp_unpack_dir_obj.name)

            self._compressed_file_extractor.extract(
                build_from_dist.path, unpack_root
            )

            wheel_path = db.DistBuilder(
                distribution="wheel",
                source_dir=unpack_root / self._unpacked_src_dir_name,
                output_directory=self._construction_dir.orig_pkg_dists,
            ).build()

            temp_unpack_dir_obj.cleanup()

        completed_msg = f"\tBuilt wheel {wheel_path.name}"
        logging.getLogger(f"std_out.{__name__}").info(completed_msg)



from typing import List, Union

import srepkg.repackager_data_structs as re_ds
import srepkg.repackager_interfaces as re_int
import srepkg.orig_src_preparer_interfaces as osp_int


class OrigSrcPreparer(re_int.OrigSrcPreparerInterface):

    def __init__(
        self,
        retriever_provider: List[
            Union[
                osp_int.RemotePkgRetrieverInterface,
                osp_int.DistProviderInterface,
            ]
        ],
        receiver: osp_int.ManageableConstructionDir,
    ):
        self._retriever_provider = retriever_provider
        self._receiver = receiver

    def prepare(self) -> re_ds.ConstructionDirSummary:
        for component in self._retriever_provider:
            component.run()
        orig_pkg_summary = self._receiver.finalize()
        # self._receiver.settle()

        return orig_pkg_summary



"""
Solo Re-Package (srepkg)
=========================

srepkg is a Python package that wraps an isolation layer around the
source code of other Python packages. When a package that has been
“re-packaged” by srepkg is installed using a pip install command from
an existing Python environment, a new virtual environment is
automatically created. The original package and its dependencies are
installed in this newly created virtual environment to avoid the
possibility of any dependency-conflicts, but the original package’s
command line entry points are still accessible from the pre-existing
environment.
"""



import srepkg.repackager_interfaces as rep_int


class Repackager:
    """
    Repackages original package with isolation layer.
    """
    def __init__(
        self,
        srepkg_command: rep_int.SrepkgCommand,
        service_class_builder: rep_int.ServiceBuilderInterface,
    ):
        self._srepkg_command = srepkg_command
        self._service_class_builder = service_class_builder

    def repackage(self):
        construction_dir_summary = (
            self._service_class_builder.create_orig_src_preparer().prepare()
        )

        srepkg_builder = self._service_class_builder.create_srepkg_builder(
            construction_dir_summary=construction_dir_summary
        )
        srepkg_builder.build()



"""
Contains class for retrieving arguments from command line.
"""

import argparse
from pathlib import Path
import srepkg.error_handling.custom_exceptions as ce
import srepkg.repackager_interfaces as rep_int


class SrepkgCommandLine(rep_int.SrepkgCommandInterface):
    """
    Retrieves arguments from command line.
    """
    def __init__(self):
        self._parser = argparse.ArgumentParser()

    def _attach_args(self):
        """
        Attaches command line arguments to parser.

        Returns:
            None
        """
        self._parser.add_argument(
            "orig_pkg_ref",
            type=str,
            help="A reference to the original package to be repackaged. Can "
            "be a local path to the directory where a package's setup.py "
            "or pyproject.toml resides, a  PyPI package name, or a Github"
            " repo url.",
        )

        self._parser.add_argument(
            "-g",
            "--git_ref",
            type=str,
            nargs="?",
            action="store",
            help="A git branch name, tag name, or commit SHA that determines "
            "the original package commit to use (if ORIG_PKG_REF is a "
            "git repo). Defaults to: HEAD of the default branch for a "
            "remote Github repo, and the currently checked out branch "
            "for a local repo.",
        )

        self._parser.add_argument(
            "-r",
            "--pypi_version",
            type=str,
            nargs="?",
            action="store",
            help="Original package version to use (if ORIG_PKG_REF is a PyPI"
            " package). Defaults to the latest PyPI package.",
        )

        self._parser.add_argument(
            "-n",
            "--srepkg_name",
            type=str,
            nargs="?",
            action="store",
            help="Name to be used for repackaged package. Default is "
            "<{ORIGINAL_PACKAGE_NAME}srepkg>",
        )

        self._parser.add_argument(
            "-c",
            "--construction_dir",
            type=str,
            nargs="?",
            action="store",
            help="Directory where non-compressed repackage will be built and "
            "saved. If not specified, srepkg is built in a temp "
            "directory that gets deleted after wheel and or sdist archives"
            "have been created.",
        )

        self._parser.add_argument(
            "-d",
            "--dist_out_dir",
            type=str,
            nargs="?",
            action="store",
            help="Directory where srepkg wheel and or sdist archives are "
            "saved. Default is under relative path ./srepkg_dist which "
                 "gets created if it does not already exist.",
        )

        self._parser.add_argument(
            "-f",
            "--logfile_dir",
            type=str,
            nargs="?",
            action="store",
            help="Directory to write srepkg log file to. Default behavior is "
            "to write log to file in temporary directory that is "
            "automatically deleted at end of execution.",
        )

    @staticmethod
    def _set_defaults(args_namespace):
                
        if args_namespace.dist_out_dir is None:
            args_namespace.dist_out_dir = Path.cwd() / "srepkg_dists"
        
    
    def get_args(self, *args) -> rep_int.SrepkgCommand:
        """

        Args:
            *args (): Entry point arguments, typically from command line.

        Returns:
            a SrepkgCommand instance

        """
        self._attach_args()
        args_namespace = self._parser.parse_args(*args)
        self._set_defaults(args_namespace=args_namespace)
        if args_namespace.git_ref and args_namespace.pypi_version:
            raise ce.PkgVersionWithCommitRef(
                commit_ref=args_namespace.git_ref,
                pkg_version=args_namespace.pypi_version,
            )

        return rep_int.SrepkgCommand(**vars(args_namespace))



import abc


class RemotePkgRetrieverInterface(abc.ABC):

    # @abc.abstractmethod
    # def retrieve(self):
    #     pass

    @abc.abstractmethod
    def run(self):
        pass


class DistProviderInterface(abc.ABC):

    # @abc.abstractmethod
    # def provide(self):
    #     pass

    @abc.abstractmethod
    def run(self):
        pass


class ManageableConstructionDir(abc.ABC):

    @abc.abstractmethod
    def finalize(self):
        pass

    @abc.abstractmethod
    def settle(self):
        pass



"""
Contains the srepkg entry point.
"""

import srepkg.command_input as ci
import srepkg.logging_initializer as lgr
import srepkg.repackager as rep
import srepkg.service_builder as scb


def main(*args) -> None:
    """

    Args:
        *args (): entry point args, typically from command line

    Returns:
        None
    """
    srepkg_command = ci.SrepkgCommandLine().get_args(*args)

    logger_initializer = lgr.LoggingInitializer(
        logfile_dir=srepkg_command.logfile_dir
    )
    logger_initializer.setup()

    # lgr.LoggingInitializer(logfile_dir=srepkg_command.logfile_dir).setup()

    service_class_builder = scb.ServiceBuilder(srepkg_command)

    repackager = rep.Repackager(srepkg_command, service_class_builder)
    repackager.repackage()



import subprocess
import zipfile
import tarfile
from enum import Enum, auto
from pathlib import Path
import srepkg.error_handling.custom_exceptions as ce
import srepkg.utils.logged_err_detecting_subprocess as leds
from srepkg.error_handling.custom_exceptions import UnsupportedCompressionType


class ArchiveFileType(Enum):
    TAR_GZ = auto()
    ZIP = auto()
    WHL = auto()
    UNKNOWN = auto()


class ArchiveDistType(Enum):
    SDIST = auto()
    WHEEL = auto()
    UNKNOWN = auto()


# https://stackoverflow.com/a/13044946/17979376
# https://stackoverflow.com/a/63404232/17979376
class ArchiveIdentifier:
    file_signatures = {
        b"\x1f\x8b\x08": ArchiveFileType.TAR_GZ,
        b"\x50\x4b\x03\x04": ArchiveFileType.ZIP,
    }

    dist_file_table = {
        ArchiveFileType.TAR_GZ: ArchiveDistType.SDIST,
        ArchiveFileType.ZIP: ArchiveDistType.SDIST,
        ArchiveFileType.WHL: ArchiveDistType.WHEEL,
        ArchiveFileType.UNKNOWN: ArchiveDistType.UNKNOWN,
    }

    def id_file_type(self, possible_archive: Path):
        max_len = max(len(x) for x in self.file_signatures)

        with possible_archive.open(mode="rb") as f:
            file_start = f.read(max_len)
        for signature, filetype in self.file_signatures.items():
            if file_start.startswith(signature):
                archive_filetype = filetype
                # .whl files are special case of .zip
                if (
                    filetype == archive_filetype.ZIP
                    and possible_archive.suffix == ".whl"
                ):
                    archive_filetype = filetype.WHL
                return archive_filetype

        return ArchiveFileType.UNKNOWN

    def id_dist_type(self, possible_archive: Path):
        return self.dist_file_table[self.id_file_type(possible_archive)]


class CompressedFileExtractor:

    @staticmethod
    def _extract_zip(compressed_file: Path, output_dir: Path):
        with zipfile.ZipFile(compressed_file, "r") as zf:
            zf.extractall(output_dir)

    @staticmethod
    def _extract_tar_gz(compressed_file: Path, output_dir: Path):
        with tarfile.open(compressed_file) as tf:
            tf.extractall(output_dir)

    @staticmethod
    def _extract_whl(compressed_file: Path, output_dir: Path):
        # TODO replace with leds.LoggedErrDetectingSubprocess
        # subprocess.call(
        #     ['wheel', 'unpack', str(compressed_file), '--dest',
        #      str(output_dir)])

        leds.LoggedErrDetectingSubprocess(
            cmd=[
                "wheel",
                "unpack",
                str(compressed_file),
                "--dest",
                str(output_dir),
            ],
            gen_logger_name=__name__,
            std_out_logger_name="std_out",
            std_err_logger_name="std_err",
            default_exception=ce.WheelUnpackError,
        )

    @staticmethod
    def _extract_unknown(compressed_file: Path, output_dir: Path):
        raise UnsupportedCompressionType(str(compressed_file))

    @property
    def _dispatch_method(self):
        return {
            ArchiveFileType.ZIP: self._extract_zip,
            ArchiveFileType.TAR_GZ: self._extract_tar_gz,
            ArchiveFileType.WHL: self._extract_whl,
            ArchiveFileType.UNKNOWN: self._extract_unknown,
        }

    def extract(self, compressed_file: Path, output_dir: Path):
        file_type = ArchiveIdentifier().id_file_type(compressed_file)

        self._dispatch_method[file_type](compressed_file, output_dir)


# targz_file_type = ArchiveIdentifier().id_file_type(
#     Path('/Users/duane/dproj/testproj/dist/testproj-0.0.0.tar.gz')
# )
#
# zip_file_type = ArchiveIdentifier().id_file_type(
#     Path('/Users/duane/dproj/testproj/dist/testproj-0.0.0.zip')
# )
#
# whl_file_type = ArchiveIdentifier().id_file_type(
#     Path('/Users/duane/dproj/testproj/dist/testproj-0.0.0.zip')
# )
#
# print(targz_file_type)
# print(zip_file_type)
# print(whl_file_type)



import configparser
import entry_points_txt
import tempfile
from pathlib import Path
from zipfile import ZipFile

import srepkg.repackager_data_structs as re_ds
import srepkg.error_handling.custom_exceptions as ce


class WheelEntryPointExtractor:

    def __init__(self, whl_path: Path):
        self._whl_path = whl_path

    
    
    def _extract_entry_pts_txt(self, output_path: Path) -> Path:

        with ZipFile(self._whl_path, "r") as zip_obj:
            filenames_in_whl = zip_obj.namelist()
            entry_pts_txt = [
                filename
                for filename in filenames_in_whl
                if Path(filename).name == "entry_points.txt"
            ]
            if len(entry_pts_txt) == 0:
                raise ce.NoEntryPtsTxtFile(self._whl_path)
            if len(entry_pts_txt) > 1:
                raise ce.MultipleEntryPtsTxtFiles(self._whl_path)

            zip_obj.extract(entry_pts_txt[0], output_path)

            return output_path / entry_pts_txt[0]

    
    
    @staticmethod
    def _convert_to_srepkg_builder_format(
        entry_point: entry_points_txt.EntryPoint,
    ):
        # if "-" in entry_point.name:
        #     entry_point.name = entry_point.name.replace("-", "")
        
        return re_ds.CSEntryPoint(
            command=entry_point.name,
            module=entry_point.module,
            # attr=entry_point.object
            attr=entry_point.attr,
        )

    # @staticmethod
    # def remove_dashed_from_entry_point_name(entry_pts_file: Path):
    #     config = configparser.ConfigParser()
    #     config.read(entry_pts_file)
    #     sections = config.sections()
    #     num_sections = len(sections)

    
    def get_entry_points(self):
        temp_dir = tempfile.TemporaryDirectory()
        output_file = self._extract_entry_pts_txt(Path(temp_dir.name))

        with output_file.open(mode="r") as of:
            entry_pts_set = entry_points_txt.load(of)

        if ("console_scripts" not in entry_pts_set) or (
            not entry_pts_set["console_scripts"]
        ):
            raise ce.NoConsoleScriptEntryPoints(self._whl_path)

        entry_pts_list = [
            self._convert_to_srepkg_builder_format(entry)
            for entry in entry_pts_set["console_scripts"].values()
        ]

        return re_ds.PkgCSEntryPoints(cs_entry_pts=entry_pts_list)
